#!/bin/bash
# Copyleft (C) 2016 by <tv>
#
# Génère un fichier de version pour les outils de développement
#

FICHIER_CONFIG="configuration.txt"
FICHIER_VERSION="version.txt"
LISTE_OUTILS="g++ qmake make gdb svn geany pandoc espeak mysql"
LISTE_OUTILS_1="moc qtcreator"
LISTE_OUTILS_2="sqlite3 cppunit-config doxygen"
LISTE_OUTILS_3="bouml papyrus planner"

### Divers
VERSION_BOUML="Bouml 4.23"
VERSION_PAPYRUS="Papyrus 1.1.3"
VERSION_PLANNER="Planner 0.14.5"
####################################

### Informations
NAME=$(basename $0)
VERSION="1.0"
SUMMARY=""
LICENCE="GPL"
AUTEUR="Thierry Vaira <tvaira@free.fr>"
DATE="2016"
SECTION="BTS SN-IR"
####################################

### Outils
CAT="cat"
SED="sed"
INSTALL_FILE="install -m 644"
INSTALL_PROGRAM="install -m 755"
DEL_FILE="rm -f"
DEL_DIR="rmdir"
CHK_DIR_EXISTS="test -d"
CHK_FILE_EXISTS="test -f"
MKDIR="mkdir -p"
DIRMODE="755"
MKTEMP="mktemp"
MKTEMP_FILE="mktemp -q"
###################################

function interrupt_quit() 
{
   $DEL_FILE $TMP_FILE > /dev/null 2>&1
   $DEL_FILE $fichier > /dev/null 2>&1
	#echo -e "\nInstallation annulee !"
	#exit 1
}

function continuer()
{
    MESSAGE="Continuer ?"

   if ! query "$MESSAGE" 0; then
      echo -e "\x1B[0m\n"
      exit 1      
   fi
   echo -e "\x1B[0m\n"
}

function continuer_message()
{
    MESSAGE="$1"

   if ! query "$MESSAGE" 0; then
      echo -e "\x1B[0m\n"
      exit 1      
   fi
   echo -e "\x1B[0m\n"
}

function install_tmp()
{
   # temp file
   TMPDIR="/tmp/"
   # mktemp installé ?
   if [ -x "`which ${MKTEMP} 2>&-`" ]
   then      
      TMP_FILE="$(${MKTEMP_FILE} ${TMPDIR}/$0.XXXXXX)" || exit 1
   else
      TMP_FILE="${TMPDIR}/$(basename $0).$$"
      ! $CHK_FILE_EXISTS $TMP_FILE && $TOUCH $TMP_FILE > /dev/null  2>&1 || exit 1
   fi
}

function init()
{
   distribution=`${CAT} /etc/lsb-release 2> /dev/null | grep DISTRIB_DESCRIPTION | cut -d '"' -f2`
   if [ ${#distribution} -eq 0 ]
    then
        distribution=`${CAT} /etc/os-release 2> /dev/null | grep PRETTY_NAME | cut -d '=' --fields=2 | cut -d '"' --fields=2`
   fi
   os=`uname --operating-system`
   kernel=`uname --kernel-name`
   kernel_version=`uname --kernel-release`
   processor=`uname --processor`  
   #machine=`uname --hardware-platform`
   machine=`uname --machine`
   
   exe_format=`file --separator ',' $EXECUTABLE | cut -d ',' --fields=2 | sed 's/^[ \t]*//'`
   exe_machine=`file --separator ',' $EXECUTABLE | cut -d ',' --fields=3 | sed 's/^[ \t]*//'`
   exe_dynamic=`file --separator ',' $EXECUTABLE | cut -d ',' --fields=5 | sed 's/^[ \t]*//'`
   exe_os=`file --separator ',' $EXECUTABLE | cut -d ',' --fields=6 | sed 's/^[ \t]*//'`
   exe_os=$(echo "$exe_os" | sed 's/^for //')
   
   res=$(echo "$exe_dynamic" | grep 'dynamically')
   if [ ${#res} -gt 0 ]
    then
        dynamic=1
    else
        dynamic=0
   fi
}

function list_so()
{
   echo -e "Recherche des dependances sur les bibliotheques\n"
    
   fichier="./.liste.txt"
   if [ -f "$fichier" ]
   then
    rm -f "$fichier" 2> /dev/null
    #rm -f "./.liste_lib.txt" 2> /dev/null
   fi
   #liste=`ldd $EXECUTABLE | cut -d '=' --fields=1 | grep -v '0x' | sed 's/^[ \t]*//' | sed 's/[ \t]*$//'`
   liste=`ldd $EXECUTABLE | grep -i -v "not found" | grep -i -v "linux-vdso" | awk '{print $1}' | sed 's/\(^.*\).*$/\1/'`
   non_trouve=0
   for so in $liste
   do    
    dep=$(dpkg -S "$so" 2> /dev/null | grep -v 'i386' | grep -v dev | grep -v dbg | head -1 | cut -d ':' --fields=1)
    if [ -n "$dep" ]
    then
        echo "$so => $dep"
        echo "$dep" >> "$fichier"
    else
        echo "$so => aucun paquet trouve"
        #echo "$so" >> "./.liste_lib.txt"
        non_trouve=1
    fi    
  done
  if [ -f "$fichier" ]
  then
    uniq -u "$fichier" 1> /dev/null
    #cat "$fichier"
  fi
  echo ""
  liste_nontrouve=`ldd $EXECUTABLE | grep -i "not found" | awk '{print $1}' | sed 's/\(^.*\).*$/\1/' | sed 's/^[ \t]*//' | sed 's/[ \t]*$//'`
  if [ ${#liste_nontrouve} -eq 0 ]
    then
        if [ $non_trouve -eq 0 ]
        then
            echo -e "\x1B[49;32;1mToutes les dependances sont resolues par des paquets\x1B[0m"
        else
            echo -e "\x1B[49;35;1mToutes les dependances sont resolues\x1B[0m"
        fi
        echo ""        
    else
        echo -e "Dependances non resolues"
        echo -e "\x1B[49;31;1m"
        echo "$liste_nontrouve"
        echo -e "\x1B[0m"
  fi
}

function list_version()
{
   rm -rf $FICHIER_VERSION
    
   echo -e "Version des outils pour $EXECUTABLE\n" >> "$FICHIER_VERSION"
   
   for outil in $LISTE_OUTILS
   do    
    MESSAGE="$outil ?"
    if ! query "$MESSAGE" 0; then
        echo -e "\x1B[0m"
    else
        version=$($outil --version 2> /dev/null | head -1) 
        if [ ${#version} -gt 0 ]
        then
            echo "$outil => $version"
            echo "$version"  >> "$FICHIER_VERSION"
        else
            echo "$outil => ???"
        fi
        echo -e "\x1B[0m"
    fi        
  done  
  for outil in $LISTE_OUTILS_1
   do    
    MESSAGE="$outil ?"
    if ! query "$MESSAGE" 0; then
        echo -e "\x1B[0m"
    else
        $outil -version 2> .version.txt
        version=$(cat .version.txt 2> /dev/null | sed '/^$/d' | grep -i -v 'cannot' | head -1)
        if [ ${#version} -gt 0 ]
        then        
            echo "$outil => $version"
            echo "$version"  >> "$FICHIER_VERSION"
        else
            echo "$outil => ???"
        fi
        echo -e "\x1B[0m"
    fi        
  done  
  rm -f .version.txt
  for outil in $LISTE_OUTILS_2
   do    
    MESSAGE="$outil ?"
    if ! query "$MESSAGE" 0; then
        echo -e "\x1B[0m"
    else        
        version=$($outil --version 2> /dev/null | head -1) 
        if [ ${#version} -gt 0 ]
        then
            echo "$outil => $version"
            echo "$outil $version"  >> "$FICHIER_VERSION"
        else
            echo "$outil => ???"
        fi
        echo -e "\x1B[0m"
    fi        
  done  
  continuer_message "Voulez-vous ajouter les versions des outils $LISTE_OUTILS_3"
  # LISTE_OUTILS_3="qtcreator bouml papyrus planner"  
  for outil in $LISTE_OUTILS_3
   do    
    MESSAGE="$outil ?"
    if ! query "$MESSAGE" 0; then
        echo -e "\x1B[0m"
    else
        case $outil in
        bouml)      echo "$outil => $VERSION_BOUML"
                    echo "$outil $VERSION_BOUML"  >> "$FICHIER_VERSION"
                    ;;
        papyrus)    echo "$outil => $VERSION_PAPYRUS"
                    echo "$outil $VERSION_PAPYRUS"  >> "$FICHIER_VERSION"
                    ;;
        planner)    echo "$outil => $VERSION_PLANNER"
                    echo "$outil $VERSION_PLANNER"  >> "$FICHIER_VERSION"
                    ;;
        esac         
        echo -e "\x1B[0m"
    fi        
  done  
  
  echo "" >> $FICHIER_VERSION
  date=$(date)
  MESSAGE="\ngénèré le $date"
  echo -e $MESSAGE >> $FICHIER_VERSION
    
  echo -e "Fichier \x1B[49;31;1m$FICHIER_VERSION\x1B[0m génèré le $date\n" 
}

function query()
{
    QUESTION=$1
    DEFAULT=$2  

    echo -n "$QUESTION"
    if [ 0 -eq $DEFAULT ]; then
    echo -n " [O/n/q] "
    else
    echo -n " [o/N/q] "
    fi
    read -s -n 1 R <&1
    echo

    [ "$R" = "O" -o "$R" = "o" ] && return 0
    [ "$R" = "N" -o "$R" = "n" ] && return 1
    [ "$R" = "Q" -o "$R" = "q" ] && echo -e "\x1B[0m" && exit 1
    
    return $DEFAULT
}

function say_hello()
{
   $CLEARSCR

   echo -e "\x1B[49;34;1m
==========================================================
Génère un fichier de version pour les outils

GNU GPL (C) $DATE $AUTEUR $SECTION
==========================================================\x1B[0m\n"

   echo -e "Informations sur l'ordinateur"
   MESSAGE="
Distribution : $distribution
OS : $os
Noyau : $kernel
Version : $kernel_version
Machine : $machine"
   echo -e "\x1B[49;34;1m$MESSAGE\x1B[0m\n"

taille=$(ls -la $EXECUTABLE | awk '{print $5}')
taille_si=$(ls -lah --si $EXECUTABLE | awk '{print $5}')
taille_h=$(ls -lah -s $EXECUTABLE | awk '{print $6}')

   echo -e "Informations sur l'exécutable" 
   if [ $dynamic -eq 1 ]
   then
    EXE=$(basename $EXECUTABLE)
    MESSAGE="
Nom : $EXE
Taille : $taille octets ($taille_si/$taille_h)
Format : $exe_format
Machine : $exe_machine
Edition des liens : dynamique
OS : $exe_os"
   else
    MESSAGE="
Format : $exe_format
Machine : $exe_machine
Edition des liens : statique
OS : $exe_os"
   fi
   echo -e "\x1B[49;34;1m$MESSAGE\x1B[0m\n"
}

function say_lib()
{
    MESSAGE=$(cat "$fichier" | grep -i -E 'qt|qwt')
    if [ ${#MESSAGE} -gt 0 ]
    then
        echo -e "Liste des paquets Qt nécessaires\n"   
        echo -e "\x1B[49;34;1m$MESSAGE\x1B[0m\n" 
    else
        echo -e "Liste des paquets Qt nécessaires\n"   
        echo -e "\x1B[49;34;1mAucun\x1B[0m\n"     
    fi
    
    MESSAGE=$(cat "$fichier" | grep -i -v 'qt' | grep -i -v 'qwt')
    if [ ${#MESSAGE} -gt 0 ]
    then
        echo -e "Liste des autres paquets nécessaires\n"   
        echo -e "\x1B[49;34;1m$MESSAGE\x1B[0m\n"
    fi
}

function save_infos()
{
    rm -rf $FICHIER_CONFIG
    
    MESSAGE="Informations sur l'ordinateur\n\n
Distribution : $distribution\n
OS : $os\n
Noyau : $kernel\n
Version : $kernel_version\n
Machine : $machine\n"
    echo -e $MESSAGE >> $FICHIER_CONFIG

    EXE=$(basename $EXECUTABLE)
    MESSAGE="Informations sur l'exécutable\n\n
Nom : $EXE\n
Taille : $taille octets ($taille_si/$taille_h)\n
Format : $exe_format\n
Machine : $exe_machine\n
OS : $exe_os\n"
    echo -e $MESSAGE >> $FICHIER_CONFIG
   
    MESSAGE=$(cat "$fichier" | grep -i -E 'qt|qwt' | tr '\n' ' ')
    if [ ${#MESSAGE} -gt 0 ]
    then
        echo -e "Liste des paquets Qt nécessaires\n" >> $FICHIER_CONFIG
        echo -n " " >> $FICHIER_CONFIG
        echo "$MESSAGE" >> $FICHIER_CONFIG
        echo "" >> $FICHIER_CONFIG
    else
        echo -e "Liste des paquets Qt nécessaires\n" >> $FICHIER_CONFIG
        echo -n " " >> $FICHIER_CONFIG
        echo -e "Aucun\n" >> $FICHIER_CONFIG
        echo "" >> $FICHIER_CONFIG
    fi
    
    MESSAGE=$(cat "$fichier" | grep -i -v 'qt' | grep -i -v 'qwt' | tr '\n' ' ')
    if [ ${#MESSAGE} -gt 0 ]
    then
        echo -e "Liste des autres paquets nécessaires\n" >> $FICHIER_CONFIG
        echo -n " " >> $FICHIER_CONFIG
        echo -n "$MESSAGE" >> $FICHIER_CONFIG
    fi
    echo "" >> $FICHIER_CONFIG
    date=$(date)
    MESSAGE="\ngénèré le $date"
    echo -e $MESSAGE >> $FICHIER_CONFIG
    
    echo -e "Fichier \x1B[49;31;1m$FICHIER_CONFIG\x1B[0m génèré le $date\n" 
    #echo "Fichier $FICHIER_CONFIG genere le $date"
    #echo ""
}


function verif_size()
{
   DESTDIR="$1"
   shift
   BASEDIR="$1"
   shift
   basedir="${BASEDIR-pwd}"
   totalSizeKB=0
   for file in ${*}
   do
        fileSizeKB=0
        name=$(echo $basedir/$file | $SED -e 's/[^[:alnum:]\.\_\-\/]//g')
        $CHK_FILE_EXISTS $name && fileSizeKB=`ls -l $name | awk '{print $(NF-3)}' | tail -1`
        totalSizeKB=`expr $totalSizeKB + $fileSizeKB`
   done
   totalSizeKB=`expr $totalSizeKB / 1024`

   echo "Espace total requis : $totalSizeKB kB"
   echo -n "Espace disponible   : "
   freespace=`df -k $DESTDIR | tail -1 | awk '{print $(NF-2)}'`
   echo "$freespace kB"
   if [ $totalSizeKB -gt $freespace ] ; then
      return 1
   fi
   return 0
}

################################################################################
#	Main
################################################################################

# Initialisation
trap interrupt_quit 1 2 3 9 15 EXIT ERR

# Il faut un et un seul argument
if [ $# -ne "1" ]
then
    echo "Usage: $(basename $0) <executable>"
    exit 1
fi

EXECUTABLE=$1

# Il faut un fichier executable ?
if [ ! -x "$EXECUTABLE" ]
  then    
    echo "Le fichier $EXECUTABLE n'est pas un fichier executable"
    exit 1
fi

# Il faut un binaire executable ?
RES=`file "$EXECUTABLE" | grep 'LSB executable'`
if [ -z "$RES" ]
then
    echo "Le fichier $EXECUTABLE n'est pas un binaire executable"
    exit 1
fi

init

say_hello

#continuer

#list_so

#say_lib

#save_infos

continuer_message "Voulez-vous ajouter les versions des outils"

list_version

exit 0
