#ifndef BASEDEDONNEES_H
#define BASEDEDONNEES_H

#include <QObject>
#include <QtSql/QtSql>
#include <QSqlDatabase>
#include <QMutex>

#ifdef DEBUG_BASEDEDONNEES
    //#define DEBUG_BASEDEDONNEES_1
#endif

/**
 * @brief Classe singleton + ressource critique protégée par un mutex
 *
 * @class BaseDeDonnees basededonnees.h "basededonnees.h"
 *
 * @warning Il faut ajouter QT += sql au fichier .pro
 */
class BaseDeDonnees : public QObject
{
    Q_OBJECT
    public:
        /**
         * @brief
         *
         * @fn getInstance
         * @return BaseDeDonnees
         */
        static BaseDeDonnees* getInstance();
        /**
         * @brief
         *
         * @fn detruireInstance
         */
        static void detruireInstance();

        /**
         * @brief
         *
         * @fn connecter
         * @param nomReferentiel QString
         * @return bool
         */
        bool connecter(QString nomReferentiel = "referentiel-sai.sqlite");
        /**
         * @brief Pour les requêtes SQL UPDATE, INSERT et DELETE
         *
         * @fn executer
         * @param requete QString
         * @return bool
         */
        bool executer(QString requete); /* pour : UPDATE, INSERT et DELETE */
        /* uniquement pour : SELECT (enregistrement -> champ) */
        /**
         * @brief Requête SQL SELECT pour récupérer un seul champ de plusieurs enregistrements : le champ des différents enregistrements est stocké dans un QVector de QString
         *
         * @fn recuperer(QString, QVector<QString> &)
         * @param requete QString
         * @param donnees QVector<QString>
         * @return bool
         */
        bool recuperer(QString requete, QVector<QString> &donnees); // 1..* -> 1
        /**
         * @brief Requête SQL SELECT pour récupérer plusieurs champs de plusieurs enregistrements : les différents champs des différents enregistrements sont stockés dans un QVector de QStringList
         *
         * @fn recuperer(QString, QVector<QStringList> &)
         * @param requete QString
         * @param donnees QVector<QStringList>
         * @return bool
         */
        bool recuperer(QString requete, QVector<QStringList> &donnees); // 1..* -> 1..*
        /**
         * @brief Requête SQL SELECT pour récupérer un seul champ d’un seul enregistrement : le champ est stocké dans un QString
         *
         * @fn recuperer(QString, QString &)
         * @param requete QString
         * @param donnees QString
         * @return bool
         */
        bool recuperer(QString requete, QString &donnees); // 1 -> 1
        /**
         * @brief Requête SQL SELECT pour récupérer plusieurs champs d’un seul enregistrement : les différents champs sont stockés dans un QStringList
         *
         * @fn recuperer(QString, QStringList &)
         * @param requete QString
         * @param donnees QStringList
         * @return bool
         */
        bool recuperer(QString requete, QStringList &donnees); // 1 -> 1..*

    private:
/**
 * @brief
 *
 * @fn BaseDeDonnees
 */
        BaseDeDonnees();
        /**
         * @brief
         *
         * @fn ~BaseDeDonnees
         */
        ~BaseDeDonnees();
        static BaseDeDonnees*  baseDeDonnees; /**< TODO */
        static int             nbAcces; /**< TODO */
        QSqlDatabase           db; /**< TODO */
        QMutex                 mutex; /**< TODO */
};

#endif // BASEDEDONNEES_H
