#ifndef IHM_H
#define IHM_H

#include <QtGui>
#include <QTableWidget>

#include "qmapcontrol.h"


using namespace qmapcontrol;

#define DEBUG_IHM

/**
 * @brief  pour le QTableWidget informationsFlotte
 *
 * @enum ColonneInformationsFlotte
 */
typedef enum { COLONNE_ETAT = 0, COLONNE_NUMERO_BUS, COLONNE_NUMERO_LIGNE, COLONNE_NOM_CONDUCTEUR, COLONNE_DESTINATION, COLONNE_NOM_ARRET, COLONNE_PROCHAIN_ARRET, COLONNE_AVANCE_RETARD, COLONNE_VITESSE, COLONNE_HORODATAGE, COLONNE_NUMERO_SERVICE, NB_COLONNES } ColonneInformationsFlotte;

/**
 * @brief  pour le QVector<QStringList> services
 *
 * @enum ChampService
 */
typedef enum { SERVICE_ID_SERVICE, SERVICE_CODE_CONDUCTEUR, SERVICE_ID_VEHICULE, SERVICE_NOM_CONDUCTEUR , SERVICE_DATE_DEBUT, SERVICE_HEURE_DEBUT, SERVICE_DATE_FIN, SERVICE_HEURE_FIN, SERVICE_EFFECTUE} ChampService;

/**
 * @brief pour les trames
 *
 * @enum Localisation
 */
typedef enum {DATE_DEBUT, HEURE_DEBUT, IDBUS, ID_SERVICE, ID_LIGNE, ID_ITINERAIRE, LATITUDE, LAT_ORI, LONGITUDE, LONG_ORI, DIRECTION, VITESSE} Localisation;


/**
 * @brief pour le QTableWidget suivisCourse
 *
 * @enum ColonneSuivis
 */
typedef enum { COLONNE_SEQUENCE = 0, COLONNE_SERVICE_ARRET, COLONNE_SERVICE_HEURE_DEBUT_THEORIQUE, COLONNE_SERVICE_DATE_DEBUT_REEL, COLONNE_AVANCE, COLONNE_SERVICE_HEURE_ARRIVE_THEORIQUE, COLONNE_SERVICE_HEURE_ARRIVE_REEL,COLONNE_TEMPS_ARRIVE} ColonneSuivis;

/**
 * @brief pour le QVector<QStringList> suivi
 *
 * @enum ChampSuivi
 */
typedef enum { SEQUENCE, SERVICE_ARRET, SERVICE_HEURE_DEBUT_THEORIQUE, SERVICE_HEURE_ARRIVE_THEORIQUE } ChampSuivi;


/**
 * @brief Pour les configurations de la carte
 *
 * @struct ParametresCarte
 *
 */
typedef struct
{
    double latitude; /**< longitude du centre de la ville */
    double longitude; /**< latitude du centre de dans la ville  */
    int zoom; /**< zoom prédefini pour voir la carte */
    int largeur; /**< largeur du widget pour la carte */
    int hauteur; /**< hauteur du widget pour la carte */
} ParametresCarte;


class BaseDeDonnees;

class Serveur;

/**
 * @brief Class
 *
 * @class IHM ihm.h "ihm.h"
 *
  * \author \a Rialy Farel Dira <rialy23@gmail.com>
 */
class IHM : public QWidget
{
    Q_OBJECT

public:
/**
 * @brief Contructeur par défaut
 *
 * @fn IHM( QWidget *parent = 0 )
 * @param parent
 */
    IHM( QWidget *parent = 0 );
    /**
     * @brief Le destructeur de la classe
     *
     * @fn ~IHM
     */
    ~IHM();

    /**
     * @brief Pour afficher les services de la journée
     *
     * @fn afficherServices
     * @param services
     */
    void afficherServices(QVector<QStringList> services);
    /**
     * @brief Pour afficher l'arrêt en cours et le prochain arret
     *
     * @fn afficherServicesArret
     * @param arret
     */
    void afficherServicesArret(QStringList arret);
    /**
     * @brief Pour afficher la vitesse la direction et l'état du service
     *
     * @fn afficherServicesVitesse
     * @param vitesse
     */
    void afficherServicesVitesse(QStringList vitesse);

    /**
     * @brief Pour afficher le suivi d'une course
     *
     * @fn afficherSuivi
     * @param idService
     * @param suivi
     */
    void afficherSuivi(QString idVehicule, QVector<QStringList> suivi);
    /**
     * @brief Pour afficher l'heure d'arrivée réel et départ réel et l'avance retard
     *
     * @fn afficherSuiviHeure
     * @param idVehicule
     * @param arret
     */
    void afficherSuiviHeure(QString idVehicule, QStringList arret);
    /**
     * @brief Pour actualiser les positions du bus sur la carte
     *
     * @fn actualiserLocalisation
     * @param localisation
     */
    void actualiserLocalisation(QStringList localisation);
    /**
     * @brief Pour afficher dans la partie journalisation
     *
     * @fn journaliser
     * @param message
     */
    void journaliser(const QString &message);

private:
    QVector<QStringList>     services; /**< Conteneur pour stocker la liste des services */
    QVector<QStringList>     sivMessages; /**< Conteneur pour stocker les messages pour chaque SIV */
    QMap<QString,QString>    itinerairesBus; /**< Conteneur pour stocker les itineraires pour les SIV  */
    QMap<QString,QVector<QStringList> > arrets; /**< Pour stocker les arrets */
    QString                  idVehiculeSelectionne; /**< Pour stocker le vehicule selectionné */
    BaseDeDonnees           *bdd; /**< la relation avec la classe BaseDeDonnees */
    QString                  nomReferentiel; /**< Pour stocker le nom du referentiel qu'on utilise */
    Serveur                 *serveur; /**< la relation avec la classe Serveur */
    double                  retard; /**< Avance retard */
    bool                    arretclic;
    QTimer                  *timerArret;
    // Widgets
    QTableWidget            *informationsFlotte; /**< Pour le tableau de Service   */
    QTableWidget            *suivisCourse; /**< Pour le tableau pour le suivie d'une course */
    QLabel                  *labelInformationsFlotte; /**< Pour contenir le nom du tableau Service */
    QLabel                  *labelsuivisCourse; /**< Pour contenir le nom du tableau Suivis */
    QTextEdit               *journal; /**< Pour la journalisation */
    QLabel                  *nomClicCarte; /**< le nom de l'élement sur lequel on a cliqué sur la carte */
    QPushButton             *bDemarrer; /**< relation avec le bouton demarrer */
    QPushButton             *bArreter; /**< relation avec le bouton Arreter */


    // Maps
    ParametresCarte          parametresCarte; /**< Pour les parametres prédefini de la carte */
    MapControl              *mc; /**< La carte */
    TileMapAdapter          *mapadapter; /**< Tuile pour la carte qu'on va utiliser */
    Layer                   *l; /**< Calque */
    Layer                   *layerParcours; /**< Les calques pour les itineraires */
    QMap<QString, Layer*>   mapParcours; /**< Conteneur pour stocker les Calques pour chaques  idVehicules */

    Layer                   *layerarret;
    Layer                   *layerBus; /**< Calque pour le bus */
    QMap<QString, Layer*>                   mapLayer; /**< Contient les calques pour les arrets */
    QMap<QString, QList<Point*> >           mapPoint; /**< Pour contenir les idVehicules */

    /**
     * @brief
     *
     * @fn coloriserFondCellule(QTableWidgetItem *cellule, const QColor &couleur)
     * @param cellule
     * @param couleur
     */
    void coloriserFondCellule(QTableWidgetItem *cellule, const QColor &couleur);
    /**
     * @brief
     *
     * @fn coloriserFondCellule(QTableWidgetItem *cellule, const QString &couleur)
     * @param cellule
     * @param couleur
     */
    void coloriserFondCellule(QTableWidgetItem *cellule, const QString &couleur);
    /**
     * @brief
     *
     * @fn coloriserFondCellule(QTableWidget *tableWidget, int ligne, int colonne, const QColor &couleur)
     * @param tableWidget
     * @param ligne
     * @param colonne
     * @param couleur
     */
    void coloriserFondCellule(QTableWidget *tableWidget, int ligne, int colonne, const QColor &couleur);
    /**
     * @brief
     *
     * @fn coloriserFondCellule(QTableWidget *tableWidget, int ligne, int colonne, const QString &couleur)
     * @param tableWidget
     * @param ligne
     * @param colonne
     * @param couleur
     */
    void coloriserFondCellule(QTableWidget *tableWidget, int ligne, int colonne, const QString &couleur);
    /**
     * @brief
     *
     * @fn coloriserTexteCellule(QTableWidgetItem *cellule, const QColor &couleur)
     * @param cellule
     * @param couleur
     */
    void coloriserTexteCellule(QTableWidgetItem *cellule, const QColor &couleur);
    /**
     * @brief
     *
     * @fn coloriserTexteCellule(QTableWidgetItem *cellule, const QString &couleur)
     * @param cellule
     * @param couleur
     */
    void coloriserTexteCellule(QTableWidgetItem *cellule, const QString &couleur);
    /**
     * @brief
     *
     * @fn coloriserTexteCellule(QTableWidget *tableWidget, int ligne, int colonne, const QColor &couleur)
     * @param tableWidget
     * @param ligne
     * @param colonne
     * @param couleur
     */
    void coloriserTexteCellule(QTableWidget *tableWidget, int ligne, int colonne, const QColor &couleur);
    /**
     * @brief
     *
     * @fn coloriserTexteCellule(QTableWidget *tableWidget, int ligne, int colonne, const QString &couleur)
     * @param tableWidget
     * @param ligne
     * @param colonne
     * @param couleur
     */
    void coloriserTexteCellule(QTableWidget *tableWidget, int ligne, int colonne, const QString &couleur);
    /**
     * @brief
     *
     * @fn coloriserFondLigne(QTableWidget *tableWidget, int ligne, const QString &couleur)
     * @param tableWidget
     * @param ligne
     * @param couleur
     */
    void coloriserFondLigne(QTableWidget *tableWidget, int ligne, const QString &couleur);
    /**
     * @brief
     *
     * @fn coloriserTexteLigne(QTableWidget *tableWidget, int ligne, const QString &couleur)
     * @param tableWidget
     * @param ligne
     * @param couleur
     */
    void coloriserTexteLigne(QTableWidget *tableWidget, int ligne, const QString &couleur);
    /**
     * @brief
     *
     * @fn recupererServices
     * @return QVector<QStringList>
     */
    QVector<QStringList> recupererServices();
    /**
     * @brief pour recuperer le suivi d'une course
     *
     * @fn recupererSuivi
     * @param idItineraire
     * @return QVector<QStringList>
     */
    QVector<QStringList> recupererSuivi(QString idItineraire);
    /**
     * @brief Pour recuperer l'ensembles des arrets pour un itineraire
     *
     * @fn recupererArrets
     * @param idItineraire
     * @param arrets
     * @return bool
     */
    bool recupererArrets(QString idItineraire, QVector<QStringList> &arrets);
    /**
     * @brief Pour recuperer un arret dans une liste
     *
     * @fn recupererArret
     * @param nomLieu
     * @param arret
     * @return bool
     */
    bool recupererArret(QString nomLieu, QStringList &arret);
    /**
     * @brief Pour recuperer les traces d'un itineraire
     *
     * @fn recupererTraces
     * @param idItineraire
     * @param trace
     */
    void recupererTraces(QString idItineraire, QVector<QStringList> &trace);

    /**
     * @brief Pour calculer la durée entre 2 heures
     *
     * @fn calculerDuree
     * @param heureReel
     * @param heureTheorique
     * @return double
     */
    double calculerDuree(QString heureReel, QString heureTheorique);

    /**
     * @brief Pour geolocaliser le vehicule sur la carte
     *
     * @fn geocaliserVehicule
     * @param longitude
     * @param latitude
     * @param idVehicule
     */
    void geocaliserVehicule(QString longitude,QString latitude,QString idVehicule);
    /**
     * @brief Pour dessiner le tracé d'un itineraire
     *
     * @fn dessinerCourse
     * @param idVehicule
     * @param idItineraire
     */
    void dessinerCourse(QString idVehicule, QString idItineraire);
    /**
     * @brief Pour recuperer les données pour localiser la ville sur la carte
     *
     * @fn lireParametresCarte
     */
    void lireParametresCarte();
    /**
     * @brief Pour verifier si le vehicule est en service
     *
     * @fn estEnService
     * @param idVehicule
     * @param idService
     * @param idItineraire
     * @return bool
     */
    bool estEnService(QString idVehicule, QString idService, QString idItineraire);
    /**
     * @brief Pour verifier si on  est en course
     *
     * @fn estEnCourse
     * @param idService
     * @param idItineraire
     * @return bool
     */
    bool estEnCourse(QString idService, QString idItineraire);

signals:

public slots:    
    /**
     * @brief La fonction qui est appelée lorsqu'on selectionne le vehicule
     *
     * @fn selectionnerVehiculeFlotte
     * @param item
     */
    void selectionnerVehiculeFlotte(QTableWidgetItem *item);
    /**
     * @brief La fonction qui est appelée apres un double click
     *
     * @fn effacerVehiculeFlotte
     * @param item
     */
    void effacerVehiculeFlotte(QTableWidgetItem *item);
    /**
     * @brief Pour demmarrer le serveur
     *
     * @fn demarrer
     */
    void demarrer();
    /**
     * @brief Pour arreter le serveur
     *
     * @fn arreter
     */
    void arreter();
    /**
     * @brief Pour afficher le message envoyé par le SIV
     *
     * @fn afficherMessagesSIV
     * @param messageSIV
     */
    void afficherMessagesSIV(QString messageSIV);
    /**
     * @brief Pour decoder le message envoyé par le SIV
     *
     * @fn decoderMessagesSIV
     * @param messageSIV
     */
    void decoderMessagesSIV(QString messageSIV);
    /**
     * @brief La fonction qui est appelée lorsqu'on selectionne le vehicule sur la carte
     *
     * @fn selectionnerBus(Geometry*, QPoint)
     * @param layer l'id de bus
     * @param point non utilisé
     */
    void selectionnerBus(Geometry* layer, QPoint point);
    /**
     * @brief
     *
     * @fn effacerArret
     */
    void effacerArret();
};

#endif // IHM_H
