#include "serveur.h"

#include <QDebug>
#include <QMessageBox>

Serveur::Serveur( QObject *parent/*=0*/ ) : QObject( parent )
{
   //qDebug() << QString::fromUtf8("Instancie un objet QTcpServer");
   serveur = new QTcpServer(this);
}

Serveur::~Serveur()
{
   //qDebug() << QString::fromUtf8("Ferme le serveur");
}

void Serveur::demarrer()
{
    // Démarrage du serveur en écoute sur toutes les IP disponibles (= toutes les interfaces locales du serveur) et sur le port 5000
    if (!serveur->listen(QHostAddress::Any, PORT_SERVEUR))
    {
        // Si le serveur n'a pas été démarré correctement
        qDebug() << QString::fromUtf8("Le serveur n'a pas pu être démarré.") << serveur->errorString();
    }
    else
    {
        // Si le serveur a été démarré correctement
        qDebug() << QString::fromUtf8("Le serveur a été démarré sur le port ") << QString::number(serveur->serverPort());
        qDebug() << QString::fromUtf8("Adresse IP du Serveur : ") + serveur->serverAddress().toString();
        qDebug() << QString::fromUtf8("Numéro de port d'écoute du Serveur : ") + QString::number(serveur->serverPort());
        connect(serveur, SIGNAL(newConnection()), this, SLOT(connecterClient()));
    }
}

void Serveur::connecterClient()
{
    QTcpSocket *nouveauClient = serveur->nextPendingConnection();

    qDebug() << QString::fromUtf8("Un SIV s'est connecté : ") << nouveauClient->peerAddress().toString() << ":" << QString::number(nouveauClient->peerPort());
    clients << nouveauClient;

    connect(nouveauClient, SIGNAL(readyRead()), this, SLOT(recevoir()));
    connect(nouveauClient, SIGNAL(disconnected()), this, SLOT(deconnecterClient()));
    connect(nouveauClient, SIGNAL(error(QAbstractSocket::SocketError)), this, SLOT(gererErreur(QAbstractSocket::SocketError)));
}

void Serveur::deconnecterClient()
{
    // On détermine quel client se déconnecte
    QTcpSocket *client = qobject_cast<QTcpSocket *>(sender());

    if (client == 0) // Si par hasard on n'a pas trouvé le client à l'origine du signal, on arrête la méthode
        return;

    qDebug() << QString::fromUtf8("Un SIV s'est déconnecté : ") << client->peerAddress().toString() << ":" << QString::number(client->peerPort());
    clients.removeOne(client);

    client->deleteLater();
}

void Serveur::envoyer(QTcpSocket *client, QString message)
{
   qDebug() << QString::fromUtf8("Adresse IP du SIV : ") + client->peerAddress().toString();
   qDebug() << QString::fromUtf8("Numéro de port du SIV : ") + QString::number(client->peerPort());
   
   QByteArray trame; // représente le message de la couche Application
   trame.append(message);
   qint64 ecrits = -1;

   // Envoie du message
   ecrits = client->write(trame);
   switch(ecrits)
   {
      case -1 : // une erreur !
         qDebug() << QString::fromUtf8("Erreur lors de l'envoi !"); break;
      default: // envoi de n octets
         qDebug() << QString::fromUtf8("Message envoyé au SIV : ") << trame;
         //qDebug() << QString::fromUtf8("Octets envoyés : ") << ecrits;
         //qDebug() << QString::fromUtf8("Message envoyé avec succès !");
   }
}


void Serveur::recevoir()
{
   QByteArray message;
   // On détermine quel client envoie le message (recherche du QTcpSocket du client)
   QTcpSocket *client = qobject_cast<QTcpSocket *>(sender());

   if (client == 0) // Si par hasard on n'a pas trouvé le client à l'origine du signal, on arrête la méthode
           return;

   if (client->bytesAvailable() < 0)
      return;

   message = client->readAll();

   // TODO : extraire le numéro de SIV du message pour conserver la socket dans listeSIV
   //QString numSIV;
   //listeSIV[numSIV] = client;

   //qDebug() << QString::fromUtf8("Des données ont été reçues en provenance du SIV");
   //qDebug() << QString::fromUtf8("Octets reçus : ") << message.size();
   qDebug() << QString::fromUtf8("Message reçu du SIV : ") << message;

   // signaler à l'IHM que des données ont été reçues et les lui donner
   QString messageSIV(message); // QByteArray -> QString
   emit donneesRecues(messageSIV);

   //envoyer(client); // un acquittement
}

void Serveur::gererErreur(QAbstractSocket::SocketError erreur)
{
   //qDebug() << QString::fromUtf8("Une erreur s'est produite sur la socket : ") << tcpSocket->errorString();
   qDebug() << QString::fromUtf8("Erreur : ") << erreur;

}

