#ifndef SERVEUR_H
#define SERVEUR_H

#include <QObject>
#include <QtNetwork>
#include <QDebug>

#define PORT_SERVEUR 5000

/**
 * @brief La classe qui gere la connexion entre SIV et SAI
 *
 * @class Serveur serveur.h "serveur.h"
 */
class Serveur : public QObject
{
   Q_OBJECT

   public:
/**
 * @brief Constructeur
 *
 * @fn Serveur( QObject *parent=0 )
 * @param parent
 */
      Serveur( QObject *parent=0 );
      /**
       * @brief Destructeur
       *
       * @fn ~Serveur
       */
      ~Serveur();
      /**
       * @brief Pour démarrage le serveur en écoute sur toutes les IP disponibles
       *
       * @fn demarrer
       */
      void demarrer();

   private:
      QTcpServer *serveur; /**< objet serveur de type QTcpServer  */
      QList<QTcpSocket *> clients; /**< Liste des objets QTcpServer  */

      /**
       * @brief Pour envoyer les trames
       *
       * @fn envoyer(QTcpSocket *, QString)
       * @param client QTcpSocket *
       * @param message QString
       */
      void envoyer(QTcpSocket *client, QString message);

   signals:
      /**
       * @brief
       *
       * @fn donneesRecues
       * @param messageSIV
       */
      void donneesRecues(QString messageSIV);

   private slots:
      /**
       * @brief Pour connecter le client au serveur
       *
       * @fn connecterClient
       */
      void connecterClient();
      /**
       * @brief Pour deconnecter les clients du serveur
       *
       * @fn deconnecterClient
       */
      void deconnecterClient();
      /**
       * @brief Pour recevoir les messages du clients
       *
       * @fn recevoir
       */
      void recevoir();
      /**
       * @brief
       *
       * @fn gererErreur
       * @param erreur
       */
      void gererErreur(QAbstractSocket::SocketError erreur);
};

#endif
