#include "gps.h"
#include "port.h"
#include "siv.h"

#define PORT_DEFAUT "/dev/ttyUSB2"

GPS::GPS(): port(NULL), etatDemarre(false), latitudeBus(""), longitudeBus("")
{ 
    // GPS/GPRS sim 5218
    port = new Port(QLatin1String("/dev/ttyUSB2"));
    // Simualteur GPS
    //port = new Port(QLatin1String("/dev/ttyUSB1"));
#ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
#endif
}

GPS::~GPS()
{
    delete port;
#ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
#endif
}

bool GPS::demarrerGPS()
{
    if(etatDemarre == true)
        return etatDemarre;

    if(transmettre("AT+CGPS=1,1"))
    {
        QString reponse = recevoir(100);
        if(reponse.length() == 0)
            return "";
        if(!reponse.contains("OK"))
            return "";
        reponse.remove(0, 2); // enleve \r\n au début
        etatDemarre = true;
    }

    return etatDemarre;
}

bool GPS::eteindreGPS()
{
    if(etatDemarre == false)
        return etatDemarre;

    if(transmettre("AT+CGPS=0,1"))
    {
        QString reponse = recevoir(100);
        if(reponse.length() == 0)
            return "";
        if(!reponse.contains("OK"))
            return "";
        reponse.remove(0, 2); // enleve \r\n au début
        etatDemarre = false;
    }

    return etatDemarre;
}

QStringList GPS::lire()
{
    #ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << __LINE__ << QString::fromUtf8("état GPS :") << etatDemarre;
    #endif

    if(etatDemarre == false)
        demarrerGPS();

    if(etatDemarre == false)
        return QStringList("");

    if(transmettre("AT+CGPSINFO"))
    {
        QString reponse = recevoir(100);
        //QString reponse = "AT+CGPSINFO\r\r\n+CGPSINFO:3113.343286,N,12121.234064,E,250311,072809.3,44.1,0.0,0\r\nOK\r\n";
        //QString reponse = "AT+CGPSINFO\r\r\n+CGPSINFO:4356.925890,N,00448.776416,E,040516,112846.0,90.0,0,0\r\nOK\r\n";
        if(reponse.length() == 0)
        {
            //etatDemarre = false;
            return QStringList("");
        }
        if(!reponse.contains("OK"))
        {
            //etatDemarre = false;
            return QStringList("");
        }

        // Trame reponse : AT+CGPSINFO\r\r\n+CGPSINFO:4356.925890,N,00448.776416,E,040516,112846.0,90.0,0,0\r\nOK\r\n
        QStringList donnees = reponse.split('\r');
        int position = -1;
        for(int i = 0; i < donnees.size(); i++)
        {
            if(donnees.at(i).contains("CGPSINFO"))
                position = i;
        }
        if(position == -1)
            return QStringList("");

        // Avant : \n+CGPSINFO:4356.925890,N,00448.776416,E,040516,112846.0,90.0,0,0
        QString gpsinfos = donnees[position].remove(0, 11);

#ifdef DEBUG_GPS
        qDebug() << Q_FUNC_INFO << __LINE__ << donnees;
        qDebug() << Q_FUNC_INFO << __LINE__ << gpsinfos;
#endif
        // Après : 4356.925890,N,00448.776416,E,040516,112846.0,90.0,0,0

        //+CGPSINFO:[<lat>],[<N/S>],[<log>],[<E/W>],[<date>],[<UTCtime>],[<alt>],[<speed>],[<course>]
        /*   <lat>       Output format is ddmm.mmmm
         *   <log>       Output format is dddmm.mmmm
         *   <date>      Output format is ddmmyy
         *   <UTC time>  Output format is hhmmss.s
         *   <alt>       Unit is meters.
         *   <speed>     Unit is knots.
         *   <course>    Degrees.
         *
         * Exemple : +CGPSINFO:4356.925890,N,00448.776416,E,040516,112846.0,90.0,0,0
        */

        QStringList infos = gpsinfos.split(",");

#ifdef DEBUG_GPS
qDebug() << Q_FUNC_INFO << __LINE__ << infos;
#endif
        return infos;
    }

    return QStringList("");
}

void GPS::traiterInfosGPS(QStringList infos)
{
#ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << __LINE__;
    if(infos.size() > 0)
        qDebug() << "<lat> : " << infos.at(0);
    if(infos.size() > 1)
        qDebug() << "<N/S> : " << infos.at(1);
    if(infos.size() > 2)
        qDebug() << "<log> : " << infos.at(2);
    if(infos.size() > 3)
        qDebug() << "<E/W> : " << infos.at(3);
    if(infos.size() > 4)
        qDebug() << "<date> : " << infos.at(4);
    if(infos.size() > 5)
        qDebug() << "<UTC time> : " << infos.at(5);
    if(infos.size() > 6)
        qDebug() << "<alt> : " << infos.at(6);
    if(infos.size() > 7)
        qDebug() << "<speed> : " << infos.at(7);
    if(infos.size() > 8)
        qDebug() << "<course> : " << infos.at(8);
    if(infos.size() == 0)
        qDebug() << "Aucunes donnees !";
#endif
    QString latitudeD;
    QString latitudeM;
    QString longitudeD;
    QString longitudeM;

    if(infos.size() > 0)
        latitudeD = infos.at(0);
    if(infos.size() > 0)
        latitudeM = infos.at(0);
    if(infos.size() > 2)
        longitudeD = infos.at(2);
    if(infos.size() > 2)
        longitudeM = infos.at(2);

    if(latitudeD.isEmpty() || latitudeM.isEmpty() || longitudeD.isEmpty() || longitudeM.isEmpty())
        return;

#ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << __LINE__ << latitudeD << longitudeD;
#endif

    // Latitude :
    // Conversion format GPS ddmm.mmmm en degré décimal dd.dddd
    // dd + mm.mmmm/60
    // 4356.925890 —-> 43 + (56.925890/60) = 43.94876483 degre

    bool ok = false;
    double latitudeDDouble;
    double latitudeMDouble;
    double latitudeDouble;
    double longitudeDDouble;
    double longitudeMDouble;
    double longitudeDouble;
    QString longitude;
    QString latitude;

    // Pour la latitude
    // 1. Extraire dans le format ddmm.mmmm -> dd
    latitudeD.remove(2, 9);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << latitudeD;
#endif

    // 2. Extraire dans le format ddmm.mmmm -> mm.mmmm
    latitudeM.remove(0, 2);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << latitudeM;
#endif

    // 3. Convertir dd -> double
    latitudeDDouble = latitudeD.toDouble(&ok);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << latitudeDDouble;
#endif

    // 4. Convertir mm.mmmm -> double
    latitudeMDouble = latitudeM.toDouble(&ok);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << latitudeMDouble;
#endif

    // 5. Faire : 43 + (56.925890/60) = 43.94876483
    latitudeDouble = latitudeDDouble + (latitudeMDouble/60.0);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << latitudeDouble;
#endif

    // 6. Convertir les degres en QString
    latitude = QString::number(latitudeDouble);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << latitude;
#endif

    // Pour la longitude
    // 1. Extraire dans le format dddmm.mmmm -> ddd
    longitudeD.remove(3, 10);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << longitudeD;
#endif

    // 2. Extraire dans le format ddmm.mmmm -> mm.mmmm
    longitudeM.remove(0, 3);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << longitudeM;
#endif

    // 3. Convertir ddd -> double
    longitudeDDouble = longitudeD.toDouble(&ok);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << longitudeDDouble;
#endif

    // 4. Convertir mm.mmmm -> double
    longitudeMDouble = longitudeM.toDouble(&ok);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << longitudeMDouble;
#endif

    // 5. Faire : 00448.776416 4 + (48.776416/60.0) = 4.812940267
    longitudeDouble = longitudeDDouble + (longitudeMDouble/60.0);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << longitudeDouble;
#endif

    // 6. Convertir les degres en QString
    longitude = QString::number(longitudeDouble);

#ifdef DEBUG_GPS
    //qDebug() << Q_FUNC_INFO << __LINE__ << longitude;
#endif

#ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << __LINE__ << latitude << longitude;
#endif

    // 7. mise à jour de la localisation :
#ifndef SIMULATION_ET2
    if(latitude.isEmpty() || longitude.isEmpty())
        return;
    latitudeBus = latitude;
    longitudeBus = longitude;
#endif
}

int GPS::transmettre(const QString &message)
{
    QString delimiteur = "\r\n";

    if (!port->estOuvert())
    {
#ifdef DEBUG_GPS
        qDebug() << Q_FUNC_INFO << __LINE__ << QString::fromUtf8("port fermé");
#endif
        return 0;
    }

#ifdef DEBUG_GPS
        qDebug() << Q_FUNC_INFO << __LINE__ << message.toLatin1();
#endif
    int i = port->transmettre(message.toLatin1() + delimiteur.toLatin1());

    return i;
}

QString GPS::recevoir(unsigned int timeout/*=100*/)
{
    QString message = "";
    char buffer[1024+1]; // pour le fin de chaine
    int  nbOctets = 0;
    QTime t;
    int debut = 0;

    t.start();

    if (!port->estOuvert()) return "";

    if(timeout != 0)
    {
        debut = t.elapsed();
        while((nbOctets == 0) && (t.elapsed() < (int)(debut + timeout)))
        {
            nbOctets = port->getNbOctetsDisponibles();
            if(nbOctets > 1024)
                nbOctets = 1024;
            if(nbOctets > 0)
            {
                int i = port->lire(buffer, nbOctets);
                if (i != -1)
                {
                    if (buffer[i-1] == '\n' && buffer[i-2] == '\r')
                        buffer[i-2] = '\0';
                    else
                        buffer[i] = '\0';
                }
                else
                    buffer[0] = '\0';
                message += QLatin1String(buffer);
                nbOctets = 0;
            }
        }
#ifdef DEBUG_GPS
        if(message.length() == 0)
            qDebug() << Q_FUNC_INFO << QString::fromUtf8("aucune donnée reçue");
        else
            qDebug() << Q_FUNC_INFO << QString::fromUtf8("donnée reçue :") << message.toAscii().data();
            //qDebug("%s -> receive (%d) : %s", Q_FUNC_INFO, message.length(), message.toAscii().data());
        //qDebug("en %d ms", t.elapsed());
#endif

        return message;
    }
    else
    {
        while(nbOctets == 0)
        {
            nbOctets = port->getNbOctetsDisponibles();
            if(nbOctets > 1024)
                nbOctets = 1024;
        }
    }

    int i = port->lire(/*buffer, nbOctets*/);
    if (i != -1)
    {
        if (buffer[i-1] == '\n' && buffer[i-2] == '\r')
            buffer[i-2] = '\0';
        else
            buffer[i] = '\0';
    }
    else
        buffer[0] = '\0';
    message = QLatin1String(buffer);

#ifdef DEBUG_GPS
    qDebug("-> receive (%d/%d) : %s", i, nbOctets, message.toAscii().data());
#endif

    return message;
}

QString GPS::getLatitude()
{
    return latitudeBus;
}

QString GPS::getLongitude()
{
    return longitudeBus;
}
