#include "pupitreconducteur.h"
#include "tgestionconducteur.h"
#include "siv.h"
#include <iostream>

using namespace std;

/*
 * Constructeur : instancie un objet PupitreConducteur
 *
 * Paramètre(s) en entrée : port de type QString contenant le chemin vers le fichier de périphérique [valeur par défaut "/dev/qtp"]
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune
 */

PupitreConducteur::PupitreConducteur(QString port /*= "/dev/qtp"*/) : qtp(NULL), bdd(NULL), idVehicule(""), codeConducteur(""), fini(false)
{ 
    #ifdef DEBUG_QTP
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif
    #ifndef SIMULATION_PUPITRE
    qtp = new QTP(port);
    #else
    Q_UNUSED(port)
    #endif

    bdd = BaseDeDonnees::getInstance();
}

/*
 * Destructeur : libère l'objet PupitreConducteur
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune
 */

PupitreConducteur::~PupitreConducteur()
{
    #ifdef DEBUG_QTP
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif
    #ifndef SIMULATION_PUPITRE
    delete qtp;
    #endif
}

void PupitreConducteur::setIdVehicule(QString idVehicule)
{
    this->idVehicule = idVehicule;
    bdd->connecter("referentiel-" + idVehicule + ".sqlite");
}

/*
 * Rôle de la méthode : Cette méthode permet de gerer le menu
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (méthode void)
 */

int PupitreConducteur::gererMenuService(QString &donnee, int etatService, int enCourse)
{
    char touche;
    int choixService = AUCUN;
    bool etat;

    do
    {
        if(enCourse == PAS_EN_COURSE)
            afficherMenuPrincipal();

        #ifdef SIMULATION_PUPITRE
        lireTouche(&touche, 1, QTP_NOECHO);
        #else
        qtp->lireTouche(&touche, 1, QTP_NOECHO);
        #endif
        switch(touche)
        {
            case '1' :
                if(etatService == PAS_EN_SERVICE)
                {
                    codeConducteur = "";
                    etat = prendreService();
                    if(etat == true)
                    {
                        choixService = PRISE_SERVICE;
                        donnee = codeConducteur;
                    }
                    else
                    {
                        choixService = AUCUN;
                        donnee = "";
                    }
                    fini = true;
                }
                break;
            case '2' :
                if(etatService == EN_SERVICE)
                {
                    choixService = FIN_SERVICE;
                    fini = true;
                }
                else
                    afficherAucunService();
                break;
            case '3' :
                if(etatService == EN_SERVICE)
                {
                    choixService = DEMARRER_COURSE;
                    fini = true;
                }
                break;
            case '4' :
                if(etatService == EN_SERVICE)
                {
                    choixService = AFFICHER_COURSE;
                    fini = true;
                }
                break;
            /*case '5' :
                fini = true; // pour la démo
                break;*/
            default  :
                break;
        }
    }
    while(!fini);

    return choixService;
}

/*
 * Rôle de la méthode : Cette méthode permet d'afficher le menu principal
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (méthode void)
 */

void PupitreConducteur::afficherMenuPrincipal()
{
    char messages[NB_LIGNES][LG_LIGNE+1] =
    {
        {"1. PRISE DE SERVICE"}
        ,{"2. FIN DE SERVICE"}
        ,{"3. DEMARRER COURSE"}
        ,{"4. AFFICHER COURSE"}
    };

    afficherMessage(messages);
}


/*
 * Rôle de la méthode : Cette méthode permet au chauffer de s'authentifié pour son service
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucun
 */

bool PupitreConducteur::prendreService()
{
    char touche;

    saisirCodeConducteur();

    if(!verifierIdentifiantConducteur(codeConducteur))
    {
        afficherCodeFaux();
        #ifdef DEBUG_QTP
        qDebug() << "Code faux";
        #endif
        return false;
    }
    else
    {
        afficherCodeBon();
        #ifdef DEBUG_QTP
        qDebug() << "Code bon";
        #endif
        return true;
    }
}


/*
 * Rôle de la méthode : Cette méthode permet de saisir l'id du conducteur
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : code conducteur de type QString
 */

void PupitreConducteur::saisirCodeConducteur()
{
        char touche;

        char messages[NB_LIGNES][LG_LIGNE+1] =
        {
             {"*                  *"}
             ,{"  PRISE DE SERVICE  "}
             ,{"     SAISIR CODE    "}
             ,{"                    "}
        };

        afficherMessage(messages);

        do
        {
            #ifdef SIMULATION_PUPITRE
            lireTouche(&touche, 1, QTP_NOECHO);
            #else
            qtp->lireTouche(&touche, 1, QTP_NOECHO);
            #endif

            if(touche == ESC)
                break;
            else if(touche == CR)
                break;
            else
                codeConducteur += touche;
            #ifdef DEBUG_QTP
            qDebug() << codeConducteur ;
            #endif
        }
        while(touche != CR);
}


/*
 * Rôle de la méthode : Cette méthode permet de vérifié le code du conducteur
 *
 * Paramètre(s) en entrée : message de type QString
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (méthode void)
 */

bool PupitreConducteur::verifierIdentifiantConducteur(QString codeConducteur)
{
    QString code;
    QString requete = "SELECT codeConducteur FROM service WHERE codeConducteur='"+ codeConducteur + "' AND effectue=0";

    bool etat = bdd->recuperer(requete, code);

    if(etat == true)
    {
        if(code == codeConducteur)
            return true;
        else
            return false;
    }

    return false;
}

/*
 * Rôle de la méthode : Cette méthode permet de visualiser les infos d'arret
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (méthode void)
 */

bool PupitreConducteur::visualiserInformationsArret(QString idItineraire, QString numLigne, QString destination, QString prochainArret, QString numArret)
{
    QStringList informationsArret;
    QString heureDepart = "";
    QString ligne1 = "", ligne2 = "", ligne3 = "", ligne4 = "";
    char messages[NB_LIGNES][LG_LIGNE+1] =
    {
         {"                    "}
         ,{"                    "}
         ,{"                    "}
         ,{"                    "}
    };
    QTime maintenant = QTime::currentTime();
    QString heure = maintenant.toString("hh:mm");

    #ifdef DEBUG_QTP
    qDebug() << Q_FUNC_INFO << numLigne << destination << prochainArret << numArret;
    #endif

    if(numArret.length() > 0)
    {
        QString requete = "SELECT arret.heureDepart FROM arret WHERE idItineraire="+ idItineraire +" and numeroSequence="+numArret;
        bool etat = bdd->recuperer(requete, informationsArret);

        #ifdef DEBUG_QTP
        qDebug() << Q_FUNC_INFO << requete;
        #endif
        if(etat == true)
        {
            #ifdef DEBUG_QTP
            qDebug() << Q_FUNC_INFO << informationsArret << endl;
            #endif
            heureDepart = informationsArret.at(0);
        }
        else
        {
            #ifdef DEBUG_QTP
            qDebug() << Q_FUNC_INFO << "erreur requete !";
            #endif
            heureDepart = informationsArret.at(0);
        }
    }

    if(numLigne.length() > 0)
    {
        ligne1 = "LIGNE : " + numLigne + "     " + heure;
    }
    else
    {
        ligne1 = "En attente ...";
    }
    strcpy(messages[0], ligne1.toAscii().data());

    if(destination.length() < LG_LIGNE)
    {
        ligne2 = destination;
        ligne2 = nettoyer(ligne2);
    }
    else
    {
        // cf. mid()
        ligne2 = destination.mid(0, LG_LIGNE);
        ligne2 = nettoyer(ligne2);
    }
    strcpy(messages[1], ligne2.toAscii().data());

    if(prochainArret.length() < LG_LIGNE)
    {
        ligne3 = prochainArret;
        ligne3 = nettoyer(ligne3);
    }
    else
    {
        ligne3 = prochainArret.mid(0, LG_LIGNE);
        ligne3 = nettoyer(ligne3);
    }
    strcpy(messages[2], ligne3.toAscii().data());

    if(numLigne.length() > 0)
    {
        ligne4 = "Depart : " + heureDepart;
        strcpy(messages[3], ligne4.toAscii().data());
    }

    afficherMessage(messages);
    if(numLigne.length() == 0)
    {
        sleep(1);
    }

    //char touche = 0;
    //do
    /*{
        #ifdef SIMULATION_PUPITRE
        lireTouche(&touche, 1, QTP_NOECHO);
        #else
        qtp->lireTouche(&touche, 1, QTP_NOECHO);
        #endif
    }
    //while(touche != ESC);
    if(touche == ESC)
        return true;*/
    return false;
}

/*
 * Rôle de la méthode : Cette méthode permet d'afficher un message
 *
 * Paramètre(s) en entrée : message de type char
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (méthode void)
 */

void PupitreConducteur::afficherMessage(char messages[][LG_LIGNE+1])
{
    int nbLignes = NB_LIGNES;
    int i;

    #ifdef SIMULATION_PUPITRE
    for(i=0;i<nbLignes;i++)
    {
        qDebug("%s", messages[i]);
    }
    #else
    qtp->cursorOff();
    for(i=0;i<nbLignes;i++)
    {
        qtp->gotoxy(i, 0);
        qtp->clearEndofline();
        qtp->afficher(messages[i], strlen(messages[i]));
    }
    #endif
}

QString PupitreConducteur::nettoyer(QString message)
{
    QString messageNettoye;

    messageNettoye = message.replace(QString::fromUtf8("ç"), "c");
    messageNettoye = message.replace(QString::fromUtf8("â"), "a");
    messageNettoye = message.replace(QString::fromUtf8("ê"), "e");
    messageNettoye = message.replace(QString::fromUtf8("î"), "i");
    messageNettoye = message.replace(QString::fromUtf8("ô"), "o");
    messageNettoye = message.replace(QString::fromUtf8("û"), "u");
    messageNettoye = message.replace(QString::fromUtf8("à"), "a");
    messageNettoye = message.replace(QString::fromUtf8("è"), "e");
    messageNettoye = message.replace(QString::fromUtf8("é"), "e");
    messageNettoye = message.replace(QString::fromUtf8("ï"), "i");
    messageNettoye = message.replace(QString::fromUtf8("ë"), "e");

    return messageNettoye;
}

/*
 * Rôle de la méthode : Cette méthode permet d'effacer l'écran
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (méthode void)
 */

void PupitreConducteur::effacer()
{
    int nbLignes = NB_LIGNES;
    int i;

    #ifdef SIMULATION_PUPITRE
    Q_UNUSED(nbLignes)
    Q_UNUSED(i)
    //system("clear");
    #else
    qtp->cursorOff();
    for(i=0;i<nbLignes;i++)
    {
        qtp->gotoxy(i, 0);
        qtp->clearEndofline();
    }
    qtp->gotoxy(0, 0);
    qtp->cursorOn();
    #endif
}

void PupitreConducteur::afficherCodeBon()
{
    char messages[NB_LIGNES][LG_LIGNE+1] =
    {
         {"*                  *"}
         ,{"      SERVICE       "}
         ,{"        OK          "}
         ,{"                    "}
    };
    afficherMessage(messages);
    sleep(1);
}

void PupitreConducteur::afficherCodeFaux()
{
    char messages[NB_LIGNES][LG_LIGNE+1] =
    {
         {"*                  *"}
         ,{"       AUCUN        "}
         ,{"      SERVICE       "}
         ,{"                    "}
    };
    afficherMessage(messages);
    sleep(1);
}

void PupitreConducteur::afficherCourseIndisponible()
{
    char messages[NB_LIGNES][LG_LIGNE+1] =
    {
         {"*                  *"}
         ,{"   AUCUNE COURSE   "}
         ,{"     DISPONIBLE    "}
         ,{"                    "}
    };
    afficherMessage(messages);
    sleep(1);
}

void PupitreConducteur::afficherAucunService()
{
    char messages[NB_LIGNES][LG_LIGNE+1] =
    {
         {"*                  *"}
         ,{"      SERVICE       "}
         ,{"     NON ACTIF      "}
         ,{"                    "}
    };
    afficherMessage(messages);
    sleep(1);
}

void PupitreConducteur::afficherCourseDemarrer()
{
    char messages[NB_LIGNES][LG_LIGNE+1] =
    {
         {"*                  *"}
         ,{"      COURSE        "}
         ,{"     DEMARRER       "}
         ,{"                    "}
    };
    afficherMessage(messages);
    sleep(1);
}

void PupitreConducteur::afficherFinService()
{
    char messages[NB_LIGNES][LG_LIGNE+1] =
    {
         {"*                  *"}
         ,{"        FIN         "}
         ,{"      SERVICE       "}
         ,{"                    "}
    };
    afficherMessage(messages);
    sleep(1);
}

// Simulation du clavier QTP

/*
 * Rôle de la méthode : Cette méthode permet de lire une touche saisie et la convertit pour simulé un clavier QTP
 *
 * Paramètre(s) en entrée : - touche est un pointeur de type char
 *                          - nbchar de type int qui donne le nombre de caractère
 *                          - mode de type int
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : key de type char
 */

char PupitreConducteur::lireTouche(char *touche, int nbchar, int mode)
{
    Q_UNUSED(mode)

    char key;
    int lus;

    for(lus=0;lus<nbchar;lus++)
    {
        cin >> key;
        *(touche+lus) = key;
    }

    // conversion de touche (key)
    if (key == 48) // 0x30
    {
        key = 0;
    }
    if (key == 49)
    {
        key = 1;
    }
    if (key == 50)
    {
        key = 2;
    }
    if (key == 51)
    {
        key = 3;
    }
    if (key == 52)
    {
        key = 4;
    }
    if (key == 53)
    {
        key = 5;
    }
    if (key == 54)
    {
        key = 6;
    }
    if (key == 55)
    {
        key = 7;
    }
    if (key == 56)
    {
        key = 8;
    }
    if (key == 57)
    {
        key = 9;
    }
    if (key == 0x23)
    {
        key = DIESE;
    }
    if (key == 0x2A)
    {
        key = ETOILE;
    }

    return key;
}
