#ifndef PUPITRECONDUCTEUR_H
#define PUPITRECONDUCTEUR_H

#include <QtCore>
#include "qtp.h"
#include "basededonnees.h"

//#define SIMULATION_PUPITRE

#define LG_CODE 4

/**
 * @brief Classe qui gère le pupitre du conducteur
 *
 * @class PupitreConducteur pupitreconducteur.h "pupitreconducteur.h"
 *
 * @author Cyril Cambe <cambecyril@yahoo.fr>
 */
class PupitreConducteur : public QObject
{
    Q_OBJECT

private:
    QTP *qtp; /**< Un pointeur sur un objetde type QTP */
    BaseDeDonnees *bdd; /**< Un pointeur sur un objet de type BaseDeDonnees */
    QString idVehicule; /**< QString permettant d'identifié l'id véhicule */
    QString codeConducteur; /**< QString contenant le code du conducteur */
    bool fini; /**< Un bool pour savoir si la gestion est fini ou non */

    /**
     * @brief Permet de lire une touche saisis
     *
     * @fn lireTouche
     * @param touche
     * @param nbchar
     * @param mode avec un echo sur l'afficheur ou pas
     * @return char
     */
    char lireTouche(char *touche, int nbchar, int mode);
    /**
     * @brief Affiche le menu principal du pupitre
     *
     * @fn afficherMenuPrincipal
     */
    void afficherMenuPrincipal();
    /**
     * @brief Affiche le menu service et permet de prendre son service
     *
     * @fn prendreService
     * @return bool
     */
    bool prendreService();
    /**
     * @brief Permet la saisie du code conducteur
     *
     * @fn saisirCodeConducteur
     */
    void saisirCodeConducteur();
    /**
     * @brief Verifie si le code conducteur est correct avec la base de données
     *
     * @fn verifierIdentifiantConducteur
     * @param code
     * @return bool
     */
    bool verifierIdentifiantConducteur(QString code);
    /**
     * @brief Affiche un message sur le pupitre
     *
     * @fn afficherMessage
     * @param message[][]
     */
    void afficherMessage(char message[][LG_LIGNE+1]);
    /**
     * @brief permet de nettoyé un message pour les codes ASCII
     * @param message
     * @return QStrin
     */
    QString nettoyer(QString message);



public:
    /**
     * @brief Constructeur de la classe PupitreConducteur
     *
     * @fn PupitreConducteur
     * @param port
     */
    PupitreConducteur(QString port = "/dev/qtp");
    /**
     * @brief Destructeur de la classe PupitreConducteur
     *
     * @fn ~PupitreConducteur
     */
    ~PupitreConducteur();

    /**
     * @brief Permet de determiner le numéro du véhicule
     *
     * @fn setIdVehicule
     * @param idVehicule
     */
    void setIdVehicule(QString idVehicule);
    /**
     * @brief Gère le menu principal du pupitre
     *
     * @fn gererMenuService
     * @param donnee
     * @param etatService
     * @return int
     */
    int  gererMenuService(QString &donnee, int etatService, int enCourse=0);

    /**
     * @brief Efface l'écran du pupitre
     *
     * @fn effacer
     */
    void effacer();   
    /**
     * @brief Affiche que le code saisi est bon
     *
     * @fn afficherCodeBon
     */
    void afficherCodeBon();
    /**
     * @brief Affiche que le code saisi est faux
     *
     * @fn afficherCodeFaux
     */
    void afficherCodeFaux();
    /**
     * @brief Affiche quand il n'y a pas/plus de course à effectuée
     *
     * @fn afficherCourseIndisponible
     */
    void afficherCourseIndisponible();

    /**
     * @brief Affiche quand il n'y a pas de service actif
     *
     * @fn afficherAucunService
     */
    void afficherAucunService();
    /**
     * @brief Affiche quand on met fin au service
     *
     * @fn afficherFinService
     */
    void afficherFinService();
    /**
     * @brief Affiche quand la course est demarrer
     *
     * @fn afficherCourseDemarrer
     */
    void afficherCourseDemarrer();
    /**
     * @brief Affiche les informations d'une courses et arret
     *
     * @fn visualiserInformationsArret
     */
    bool visualiserInformationsArret(QString idItineraire, QString numLigne, QString destination, QString prochainArret, QString numArret);

public slots:

signals:

};

#endif // PUPITRECONDUCTEUR_H
