#include <QApplication>
#include <QDebug>
#include <QMessageBox>

#include "siv.h"
#include "tgestionconducteur.h"
#include "tinformationvoyageur.h"
#include "tacquisitionlocalisation.h"
#include "tcommunicationsai.h"

/**
 * @brief Programme principal
 *
 * @details démarre les threads de l'application
 *
 * @file siv.cpp
 */

/**
 * @brief Programme principal
 *
 * @fn main(int argc, char **argv)
 * @param argc
 * @param argv
 * @return int
 */
int main(int argc, char **argv)
{
   QApplication a(argc, argv);

   a.setApplicationName("siv");

   //qDebug() << "PID : " << (int) a.applicationPid();
   qDebug() << Q_FUNC_INFO << "SIV DEBUT" << QApplication::instance()->thread()->currentThreadId() << a.thread();

   TGestionConducteur gestionConducteur;
   QThread threadGestionConducteur;
   TInformationVoyageur informationVoyageur;
   QThread threadInformationVoyageur;
   TAcquisitionLocalisation acquisitionLocalisation;
   QThread threadacquisitionLocalisation;
   TCommunicationSAI communicationSAI;
   QThread threadCommunicationSAI;

   gestionConducteur.moveToThread(&threadGestionConducteur);
   informationVoyageur.moveToThread(&threadInformationVoyageur);
   acquisitionLocalisation.moveToThread(&threadacquisitionLocalisation);
   communicationSAI.moveToThread(&threadCommunicationSAI);
   
   QObject::connect(&threadGestionConducteur, SIGNAL(started()), &gestionConducteur, SLOT(main()));
   QObject::connect(&threadGestionConducteur, SIGNAL(finished()), &gestionConducteur, SLOT(terminer()));
   QObject::connect(&threadInformationVoyageur, SIGNAL(started()), &informationVoyageur, SLOT(main()));
   QObject::connect(&threadInformationVoyageur, SIGNAL(finished()), &informationVoyageur, SLOT(terminer()));
   QObject::connect(&threadacquisitionLocalisation, SIGNAL(started()), &acquisitionLocalisation, SLOT(main()));
   QObject::connect(&threadacquisitionLocalisation, SIGNAL(finished()), &acquisitionLocalisation, SLOT(terminer()));
   QObject::connect(&threadCommunicationSAI, SIGNAL(started()), &communicationSAI, SLOT(main()));
   QObject::connect(&threadCommunicationSAI, SIGNAL(finished()), &communicationSAI, SLOT(terminer()));

   // signaux/slots entre threads de l'application
   QObject::connect(&gestionConducteur, SIGNAL(actualiserCourse(QString)), &informationVoyageur, SLOT(actualiserCourse(QString)), Qt::DirectConnection);
   QObject::connect(&acquisitionLocalisation, SIGNAL(actualiserLocalisation(QString,QString)), &informationVoyageur, SLOT(actualiserLocalisation(QString,QString)), Qt::DirectConnection);
   QObject::connect(&gestionConducteur, SIGNAL(actualiserCourse(QString)), &acquisitionLocalisation, SLOT(demarrer(QString)), Qt::DirectConnection);
   QObject::connect(&gestionConducteur, SIGNAL(courseFinie(QString)), &acquisitionLocalisation, SLOT(arreterCourse(QString)), Qt::DirectConnection);
   QObject::connect(&informationVoyageur, SIGNAL(actualiserInformationsVoyageur(QString,QString,QString,QString,int)), &gestionConducteur, SLOT(actualiserInformationsVoyageur(QString,QString,QString,QString,int)), Qt::DirectConnection);
   #ifdef SIMULATION_ET2
   QObject::connect(&gestionConducteur, SIGNAL(actualiserCourse(QString)), &acquisitionLocalisation, SLOT(demarrerSimulationEt2(QString)), Qt::DirectConnection);
   #endif

   // démarre les threads
   threadGestionConducteur.start();
   threadInformationVoyageur.start();
   threadacquisitionLocalisation.start();
   threadCommunicationSAI.start();

   QMessageBox::information(0, QObject::tr("SIV"), QObject::tr("Cliquez sur Ok pour terminer l'application"));

   // met fin aux threads
   gestionConducteur.finir();
   threadGestionConducteur.quit();
   acquisitionLocalisation.finir();
   threadacquisitionLocalisation.quit();
   communicationSAI.finir();
   threadCommunicationSAI.quit();
   informationVoyageur.finir();
   threadInformationVoyageur.quit();   

   // attend la fin des threads
   threadGestionConducteur.wait();
   threadInformationVoyageur.wait();
   threadacquisitionLocalisation.wait();
   threadCommunicationSAI.wait();

   qDebug() << Q_FUNC_INFO << "SIV FIN" << QApplication::instance()->thread()->currentThreadId() << a.thread();

   return 0;
}
