#include "tacquisitionlocalisation.h"
#ifdef SIMULATION_ET2
#include "basededonnees.h"
#endif

TAcquisitionLocalisation::TAcquisitionLocalisation() : fini(false)
{ 
#ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
#endif
   gps = new GPS();
   demarrageLocalisation = false;
#ifdef SIMULATION_ET2
   // Simulation pour l'étudiant 2 (voir itinéaire TGestionConducteur::main())
   bdd = BaseDeDonnees::getInstance();
   qDebug() << Q_FUNC_INFO << bdd;
   bdd->connecter("referentiel-123.sqlite");
   point = 0;
   QString requete = "SELECT trace.pt_lat, trace.pt_lon  FROM trace INNER JOIN traces ON traces.idTrace=trace.idTrace WHERE traces.idItineraire='4503603922673401' ORDER BY trace.pt_sequence ASC";
   bdd->recuperer(requete, trajet);
   //qDebug() << Q_FUNC_INFO << trajet;
   etatSimulation = false;
#endif
}

TAcquisitionLocalisation::~TAcquisitionLocalisation()
{
    delete gps;
#ifdef SIMULATION_ET2
   BaseDeDonnees::detruireInstance();
#endif
#ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
#endif
}

// le corps du thread
void TAcquisitionLocalisation::main()
{
#ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
#endif

    //bool ok = gps->demarrerGPS();
    QStringList infosGPS;

    while(!fini)
    {
        if(demarrageLocalisation == true)
        {
            infosGPS = gps->lire();

            if(infosGPS.size() > 1)
                gps->traiterInfosGPS(infosGPS);
        }

        // Simulation pour l'étudiant 2 ?
        #ifdef SIMULATION_ET2
        simulerEtudiant2();
        #else
        if(infosGPS.size() > 1)
            signalerLocalisation();
        #endif

        this->msleep(1000); // période d'acquisition à définir
    }

    gps->eteindreGPS();
}

void TAcquisitionLocalisation::finir()
{
    fini = true;
    annuler();
}

void TAcquisitionLocalisation::terminer()
{
#ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
#endif
}

void TAcquisitionLocalisation::demarrer(QString idItineraire)
{
    Q_UNUSED(idItineraire)

    demarrageLocalisation = true;
    gps->demarrerGPS();
}

void TAcquisitionLocalisation::arreterCourse(QString idItineraire)
{
    Q_UNUSED(idItineraire)

    demarrageLocalisation = false;
    gps->eteindreGPS();
}

#ifdef SIMULATION_ET2
void TAcquisitionLocalisation::simulerEtudiant2()
{
    if(etatSimulation == false)
        return;

    // Simulation pour l'étudiant 2 (voir itinéaire TGestionConducteur::main())
    QString latitude;
    QString longitude;

    latitude = trajet.at(point).at(0);
    longitude = trajet.at(point).at(1);
    #ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << latitude << longitude;
    #endif

    emit actualiserLocalisation(latitude, longitude);

    this->msleep(1); // le bus se déplace
    if(point < (trajet.size()-1))
        point++;
    // Fin de la simulation pour l'étudiant 2
}

void TAcquisitionLocalisation::demarrerSimulationEt2(QString idItineraire)
{
    if(idItineraire == "4503603922673401")
        etatSimulation = true;
}
#endif

void TAcquisitionLocalisation::signalerLocalisation()
{
    QString latitude;
    QString longitude;

    // récupère la localisation
    latitude = gps->getLatitude();
    longitude = gps->getLongitude();
    #ifdef DEBUG_GPS
    qDebug() << Q_FUNC_INFO << latitude << longitude;
    #endif

    // émet la localisation
    emit actualiserLocalisation(latitude, longitude);
}
