#ifndef TACQUISITIONLOCALISATION_H
#define TACQUISITIONLOCALISATION_H

#include <QtCore>
#include "siv.h"
#include "gps.h"

#ifdef SIMULATION_ET2
class BaseDeDonnees;
#endif

/**
 * @brief Classe qui permet d'acquérir la localisation et de la diffuser
 *
 * @class TAcquisitionLocalisation tacquisitionlocalisation.h "tacquisitionlocalisation.h"
 *
 * @author Rémy Roux <remy.roux84130@gmail.com>
 */
class TAcquisitionLocalisation : public QObject
{
    Q_OBJECT

private:
    GPS *gps; /**< Pointeur sur un objet GPS */
    bool fini; /**< Un bool qui permet de mettre fin à la boucle principale du thread */
    bool demarrageLocalisation; /**< Un bool qui permet de démarrer la localisation ou pas */
    QMutex mutex; /**< Un mutex pour la gestion des temporisations */
    QWaitCondition waitCondition; /**< Pour la gestion des temporisations */
    #ifdef SIMULATION_ET2
    bool etatSimulation;
    #endif
    /**
     * @brief Méthode qui permet de réaliser des temporisations
     *
     * @fn msleep valeur de la temporisation en millisecondes
     * @param sleepMS
     */
    void msleep(unsigned long sleepMS)
    {
        waitCondition.wait(&mutex, sleepMS);
    }
    /**
     * @brief Méthode qui permet d'annuler une temporisation en cours
     *
     * @fn annuler
     */
    void annuler()
    {
        waitCondition.wakeAll();
    }
    /**
     * @brief Méthode qui permet de récupérer et de diffuser la longitude et la latitude du bus
     *
     * @fn signalerLocalisation
     */
    void signalerLocalisation();    
    #ifdef SIMULATION_ET2
    BaseDeDonnees *bdd; /**< Pointeur sur un objet bdd */
    QVector<QStringList> trajet; /**< Un QVector de QStringList des trajets */
    int point; /**< Un entier pour désigner le nombre de point */
    /**
     * @brief Méthode qui permet une simulation pour l'étudiant 2
     *
     * @fn simulerEtudiant2
     */
    void simulerEtudiant2();
    #endif

public:
/**
 * @brief Constructeur par défaut de la classe TAcquisitionLocalisation
 *
 * @fn TAcquisitionLocalisation
 */
    TAcquisitionLocalisation();
    /**
     * @brief Destrcuteur par défaut de la classe TAcquisitionLocalisation
     *
     * @fn ~TAcquisitionLocalisation
     */
    ~TAcquisitionLocalisation();

public slots:
    /**
     * @brief Méthode qui comprend le corps du thread de la classe TAcquisitionLocalisation
     *
     * @fn main
     */
    void main(); // le corps du thread
    /**
     * @brief Méthode qui permet de passer le booléen fini à l'état true
     *
     * @fn finir
     */
    void finir();
    /**
     * @brief Méthode qui permet de terminer le thread
     *
     * @fn terminer
     */
    void terminer();
    /**
     * @brief Méthode qui permet de démarrer la localisation pour une course
     *
     * @fn demarrer
     */
    void demarrer(QString idItineraire);
    /**
     * @brief Méthode qui permet d'arrêter la localisation pour une course
     *
     * @fn arreterCourse
     */
    void arreterCourse(QString idItineraire);
    #ifdef SIMULATION_ET2
    /**
     * @brief Méthode qui permet de démarrer la simulation d'un trajet pour l'étudiant 2
     *
     * @fn demarrerSimulationEt2
     */
    void demarrerSimulationEt2(QString idItineraire);
    #endif
signals:
    /**
     * @brief Émet la localisation courante du bus
     *
     * @fn actualiserLocalisation(QString, QString)
     * @param latitudeBus QString
     * @param longitudeBus QString
     */
    void actualiserLocalisation(QString latitudeBus, QString longitudeBus);
    /**
     * @brief Émet la localisation courante du bus
     *
     * @fn actualiserLocalisation(double, double)
     * @param latitudeBus double
     * @param longitudeBus double
     */
    void actualiserLocalisation(double latitudeBus, double longitudeBus);
};

#endif // TACQUISITIONLOCALISATION_H
