#ifndef TCOMMUNICATIONSAI_H
#define TCOMMUNICATIONSAI_H

#include <QtCore>

/**
 * @brief Classe qui permet de communiquer avec le SAI
 *
 * @class TCommunicationSAI tcommunicationsai.h "tcommunicationsai.h"
 *
 * @author Rémy Roux <remy.roux84130@gmail.com>
 */
class TCommunicationSAI : public QObject
{
    Q_OBJECT

private:
    bool fini; /**< Un bool qui permet de mettre fin à la boucle principale du thread */
    QMutex mutex; /**< Un mutex pour la gestion des temporisations */
    QWaitCondition waitCondition; /**< Pour la gestion des temporisations */
    /**
     * @brief Méthode qui permet de réaliser des temporisations
     *
     * @fn msleep valeur de la temporisation en millisecondes
     * @param sleepMS
     */
    void msleep(unsigned long sleepMS)
    {
        waitCondition.wait(&mutex, sleepMS);
    }
    /**
     * @brief Méthode qui permet d'annuler une temporisation en cours
     *
     * @fn annuler
     */
    void annuler()
    {
        waitCondition.wakeAll();
    }

public:
    /**
     * @brief Constructeur
     *
     * @fn TCommunicationSAI
     */
    TCommunicationSAI();
    /**
     * @brief Destructeur
     *
     * @fn ~TCommunicationSAI
     */
    ~TCommunicationSAI();

public slots:
    /**
     * @brief Méthode qui comprend le corps du thread
     *
     * @fn main
     */
    void main(); // le corps du thread
    /**
     * @brief Méthode qui permet de passer le booléen fini à l'état true
     *
     * @fn finir
     */
    void finir();
    /**
     * @brief Méthode qui permet de terminer le thread
     *
     * @fn terminer
     */
    void terminer();

signals:

};

#endif // TCOMMUNICATIONSAI_H
