#include "tdiffusionmessages.h"
#include "siv.h"

TDiffusionMessages::TDiffusionMessages() : fini(false)
{ 
    #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif
}

TDiffusionMessages::~TDiffusionMessages()
{
    #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif
}

// Le corps du thread ---------------------------------------------------------------------------------------

void TDiffusionMessages::main()
{

#ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << "demarrage" << this;
#endif
	
    lireParametres();

    // Diffusion périodique des messages

    informationsVoyageursPresentes = false;
    while(!fini)
    {        
        diffuser();

        attendrePeriode();
    }    

    #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << "arret" << this;
    #endif
}

// ----------------------------------------------------------------------------------------------------------

void TDiffusionMessages::finir()
{
    fini = true;
    annuler();
}

void TDiffusionMessages::terminer()
{
    #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif
}

/*
 * Rôle de la méthode : Lit les paramètres du projet inclut dans le fichier siv.ini
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune
 */

void TDiffusionMessages::lireParametres()
{
    // Le nom du fichier INI : nom-executable.ini
    QString fichierINI = qApp->applicationName() + ".ini"; // siv.ini

    QSettings parametres(fichierINI, QSettings::IniFormat);

    // Lecture des paramètres de configuration
    periode = parametres.value("diffusion/periode", "1000").toInt(); // en ms

    // durée
    duree = parametres.value("diffusion/duree", "2").toInt();

    // transition
    transition = parametres.value("diffusion/transition", "1").toInt();

    // vitesse
    vitesse = parametres.value("diffusion/vitesse", "250").toInt();

    // Affichage de débugage
    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << QString::fromUtf8("période : %1").arg(periode);
    qDebug() << Q_FUNC_INFO << QString::fromUtf8("duree : %1").arg(duree);
    qDebug() << Q_FUNC_INFO << QString::fromUtf8("transition : %1").arg(transition);
    qDebug() << Q_FUNC_INFO << QString::fromUtf8("vitesse : %1").arg(vitesse);
    #endif
}

/*
 * Rôle de la méthode : PErmet la diffusion des messages sur le panneau
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune
 */

void TDiffusionMessages:: diffuser()
{    
       afficherHeure();
       panneau.effacer(transition);

       afficherDate();
       panneau.effacer(transition);

    if(informationsVoyageursPresentes == true)
    {
        afficherLigne(numLigne);
        panneau.effacer(transition);

        //afficherDestination(destination);
        panneau.effacer(transition);

        afficherArret(arretEnCours, prochainArret);
        panneau.effacer(transition);
    }
}

/*
 * Rôle de la méthode : Affiche l'heure actuelle sur le panneau
 *
 * Paramètre(s) en entrée : position
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune
 */

void TDiffusionMessages::afficherHeure(int position)
{
    QDateTime heureActuelle = QDateTime::currentDateTime(); // appel méthode statique qui retourne l'heure et la date actuelle
    QString heure = heureActuelle.toString("hh:mm");

    panneau.afficher(heure, duree, position);
}

/*
 * Rôle de la méthode : Affiche la date actuelle sur le panneau
 *
 * Paramètre(s) en entrée : position
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune
 */

void TDiffusionMessages::afficherDate(int position)
{
    QDateTime dateActuelle = QDateTime::currentDateTime(); // appel méthode statique qui retourne l'heure et la date actuelle
    QString date = dateActuelle.toString("dd/MM/yyyy");

    panneau.afficher(date, duree, position);
}

/*
 * Rôle de la méthode : Affiche le numéro de la ligne
 *
 * Paramètre(s) en entrée : numLigne
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune
 */

void TDiffusionMessages::afficherLigne(QString numLigne)
{
    QString messageLigne = "Ligne : " + numLigne;

    // fixe ou défilant ?
    if(messageLigne.length() > MAX_TEXTE)
        panneau.defiler(messageLigne, vitesse);
    else
        panneau.afficher(messageLigne, duree);
}

void TDiffusionMessages::afficherDestination(QString destination)
{
    QString messageDestination = "Destination : " + destination;
    //destination = "Gare Avignon TGV";

    // fixe ou défilant ?
    if(messageDestination.length() > MAX_TEXTE)
        panneau.defiler(messageDestination, vitesse);
    else
        panneau.afficher(messageDestination, duree);
}

void TDiffusionMessages::afficherArret(QString nomArretEnCours, QString nomProchainArret)
{
    QString messageArretEnCours = "Arret : " + nomArretEnCours;
    QString messageProchainArret = "Prochain Arret : " + nomProchainArret;

    if(messageArretEnCours.length() > MAX_TEXTE)
        panneau.defiler(messageArretEnCours, vitesse);
    else
        panneau.afficher(messageArretEnCours, duree);

    if(!nomProchainArret.isEmpty()) // le bus arrive arrive au terminus
    {
        if(messageProchainArret.length() > MAX_TEXTE)
            panneau.defiler(messageProchainArret, vitesse);
        else
            panneau.afficher(messageProchainArret, duree);
    }
}

void TDiffusionMessages::attendrePeriode()
{
    //attente prochaine diffusion
    this->msleep(periode);
}

void TDiffusionMessages::actualiserInformationsVoyageur(QString numLigne, QString destination, QString arretEnCours, QString prochainArret, int numArret)
{
    Q_UNUSED(numArret)

    #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO;
    #endif

    this->numLigne = numLigne;
    this->destination = destination;
    this->arretEnCours = arretEnCours;
    this->prochainArret = prochainArret;

    informationsVoyageursPresentes = true;
}

