#include "tgestionconducteur.h"
#include "siv.h"

TGestionConducteur::TGestionConducteur() : fini(false), idVehicule(""), codeConducteur(""),idItineraire(""), enCourse(PAS_EN_COURSE), enService(PAS_EN_SERVICE), choixService(AUCUN), courseEnCours(QString("")), bdd(NULL)
{ 
    #ifdef DEBUG_QTP
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif
    lireINI();

    informationsConducteur.idItineraire = "";
    informationsConducteur.numLigne = "";
    informationsConducteur.destination = "";
    informationsConducteur.prochainArret = "";
    informationsConducteur.heureDepart = "";
    informationsConducteur.numArret = "";
    informationsConducteur.avanceRetard = 0;

    bdd = BaseDeDonnees::getInstance();
    if(!idVehicule.isEmpty())
        bdd->connecter("referentiel-" + idVehicule + ".sqlite");

    pupitreConducteur.setIdVehicule(idVehicule);
}

TGestionConducteur::~TGestionConducteur()
{
    #ifdef DEBUG_QTP
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif
}

// le corps du thread
void TGestionConducteur::main()
{
    #ifdef DEBUG_QTP
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif

    // Communication avec l'étudiant 2
    //emit actualiserCourse("4503603922673401");

    while(!fini)
    {

        #ifdef DEBUG_QTP
        qDebug() << Q_FUNC_INFO << "enService" << enService;
        #endif
        choixService = pupitreConducteur.gererMenuService(codeConducteur, enService, enCourse);
        #ifdef DEBUG_QTP
        qDebug() << Q_FUNC_INFO << "choixService" << choixService;
        #endif

        if(choixService == PRISE_SERVICE && enService == PAS_EN_SERVICE)
        {
            #ifdef DEBUG_QTP
            qDebug() << Q_FUNC_INFO << QThread::currentThreadId() <<"choixServce :" << choixService << "codeConducteur :" <<codeConducteur;
            #endif
            enService = EN_SERVICE;
            chargerService();
            emit debutService(codeConducteur);
            #ifdef DEBUG_QTP
            qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << "idService" << idService << "codeConducteur" << codeConducteur;
            #endif
        }

        if(choixService == DEMARRER_COURSE && enService == EN_SERVICE)
        {
            #ifdef DEBUG_QTP
            qDebug() << Q_FUNC_INFO << QThread::currentThreadId() <<"choixServce :" << choixService << "codeConducteur :" <<codeConducteur;
            #endif
            enCourse = EN_COURSE;
            chargerCourses();
            if(!demarrerCourse())
            {
                pupitreConducteur.afficherCourseIndisponible();
            }
            else
            {
                pupitreConducteur.afficherCourseDemarrer();
                choixService = AFFICHER_COURSE;
            }
        }

        if(choixService == FIN_SERVICE && enService == EN_SERVICE)
        {
            enCourse = PAS_EN_COURSE;
            enService = PAS_EN_SERVICE;
            terminerService();
            emit finService(codeConducteur);
        }
        if(choixService == AFFICHER_COURSE && enCourse == EN_COURSE)
        {
            pupitreConducteur.visualiserInformationsArret(informationsConducteur.idItineraire, informationsConducteur.numLigne, informationsConducteur.destination, informationsConducteur.prochainArret, informationsConducteur.numArret);
        }
    }

    pupitreConducteur.effacer();
}

void TGestionConducteur::finir()
{
    fini = true;
    annuler();
}

void TGestionConducteur::terminer()
{
    #ifdef DEBUG_QTP
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif
}

void TGestionConducteur::actualiserInformationsVoyageur(QString numLigne, QString destination, QString arretEnCours, QString prochainArret, int numeroArret)
{
    #ifdef DEBUG_QTP
    qDebug() << Q_FUNC_INFO << numLigne << destination << arretEnCours << prochainArret << numeroArret;
    #endif
    QString numArret = QString::number(numeroArret);
    informationsConducteur.numLigne = numLigne;
    informationsConducteur.destination = destination;
    informationsConducteur.prochainArret = prochainArret;
    informationsConducteur.numArret = numArret;
    pupitreConducteur.visualiserInformationsArret(informationsConducteur.idItineraire, informationsConducteur.numLigne, informationsConducteur.destination, informationsConducteur.prochainArret, informationsConducteur.numArret);
    if(informationsConducteur.prochainArret.length() == 0 && numArret > 0)
    {
        enCourse = PAS_EN_COURSE;
    }
}

void TGestionConducteur::lireINI()
{
    // Le nom du fichier INI : nom-executable.ini
    QString fichierINI = qApp->applicationName() + ".ini";

    QSettings parametres(fichierINI, QSettings::IniFormat);

    // Lecture des paramètres de configuration
    idVehicule = parametres.value("siv/bus","").toString();

    // Affichage de débugage
    qDebug()<<QString::fromUtf8("numéro de bus : %1").arg(idVehicule);
}

bool TGestionConducteur::chargerService()
{
    QString numeroService;
    QString requete = "SELECT idService FROM service WHERE codeConducteur='" + codeConducteur + "' AND effectue=0";

    bool etat = bdd->recuperer(requete, numeroService);

    if(etat == true)
    {
        idService = numeroService;

        QDateTime maintenant = QDateTime::currentDateTime();
        requete = "UPDATE service SET dateDebut='" + maintenant.toString("yyyy-MM-dd") + "',heureDebut='" + maintenant.toString("hh:mm:ss") + "' WHERE idService =" + idService;
        bool retour = bdd->executer(requete);
        if(retour == false)
        {
            #ifdef DEBUG_QTP
            qDebug() << Q_FUNC_INFO << QString::fromUtf8("erreur modification table course !");
            #endif
        }
        return true;
    }
    else
        return false;
}

bool TGestionConducteur::chargerCourses()
{
    QString requete = "SELECT course.idItineraire, nomConducteur, destination, direction, itineraire.idLigne, heureDepart, nomLong, nomCourt, description, course.effectue FROM service INNER JOIN conducteur ON conducteur.codeConducteur=service.codeConducteur INNER JOIN course ON course.idService=service.idService INNER JOIN itineraire ON itineraire.idItineraire=course.idItineraire INNER JOIN arret ON arret.idItineraire=itineraire.idItineraire INNER JOIN ligne ON ligne.idLigne=itineraire.idLigne WHERE service.idService='" + idService + "' AND course.effectue=0 AND arret.numeroSequence='0' ORDER BY heureDepart ASC";
    QVector<QStringList> lesCourses;
    bool etat = bdd->recuperer(requete, lesCourses);

    if(etat == true)
    {
        courses = lesCourses;
        #ifdef DEBUG_QTP_2
        qDebug() << Q_FUNC_INFO << courses << endl;
        #endif
        return true;
    }
    else
        return false;
}

bool TGestionConducteur::demarrerCourse()
{
    QStringList uneCourse;
    bool trouve = false;

    // avant de demarrer une course il faut mettre fin à la precedente
    terminerCourse();

    for(int i=0; (i < courses.size()) && (trouve == false); i++)
    {
        uneCourse = courses.at(i);
        #ifdef DEBUG_QTP
        qDebug() << Q_FUNC_INFO << "uneCourse :" << uneCourse << endl;
        #endif
        // non effectue ?
        if(uneCourse.at(9) == "0")
        {
            trouve = true;
        }
    }

    if(trouve == true)
    {
        courseEnCours = uneCourse;
        idItineraire = courseEnCours.at(0);
        informationsConducteur.idItineraire = idItineraire;
        //pupitreConducteur.setIdItineraire(idItineraire);
        #ifdef DEBUG_QTP
        qDebug() << Q_FUNC_INFO << "idItineraire = " << idItineraire;
        #endif

        emit actualiserCourse(idItineraire);
    }
    else
    {
        courseEnCours.clear();
        courseEnCours << "";
        #ifdef DEBUG_QTP
        qDebug() << Q_FUNC_INFO  << QString::fromUtf8("plus de course à effectuer !");
        #endif
    }

    #ifdef DEBUG_QTP
    qDebug() << Q_FUNC_INFO << "courseEncours :" << courseEnCours << endl;
    #endif

    return trouve;
}

void TGestionConducteur::terminerCourse()
{
    QString idItineraire;
    QStringList uneCourse;
    bool trouve = false;

    idItineraire = courseEnCours.at(0);

    // recherche la course à terminer
    for(int i=0; i < courses.size() && trouve == false; i++)
    {
        uneCourse = courses.at(i);
        // non effectue ?
        if(uneCourse.at(0) == idItineraire)
        {
            courses[i][9] = "1"; // course effectuee // 9 : element bdd "effectuee"
            trouve = true;
        }
    }

    if(trouve == true)
    {
        // on met à jour la base de données
        QString requete = "UPDATE course SET effectue=1 WHERE idItineraire='" + idItineraire + "'" ;
        bool retour = bdd->executer(requete);
        if(retour == false)
        {
            qDebug() << Q_FUNC_INFO << QString::fromUtf8("erreur modification table course !");
        }
        else
        {
            #ifdef DEBUG_QTP
            qDebug() << Q_FUNC_INFO << "course effectuee :" << idItineraire;
            #endif
            emit courseFinie(idItineraire);
        }
    }
}

void TGestionConducteur::terminerService()
{
    QDateTime maintenant = QDateTime::currentDateTime();

    //On termine la dernière course avant la fin de service
    terminerCourse();

    //on met à jour la base de données de fin de service
    QString requete = "UPDATE service SET effectue=1,dateFin='" + maintenant.toString("yyyy-MM-dd") + "',heureFin='" + maintenant.toString("hh:mm:ss") + "' WHERE idService ='" + idService +"'";
    bool retour = bdd->executer(requete);
    if(retour == false)
    {
        qDebug() << Q_FUNC_INFO << QString::fromUtf8("erreur modification table service !");
    }

    pupitreConducteur.afficherFinService();
}

