#include "tinformationvoyageur.h"
#include "siv.h"
#include "basededonnees.h"


/*
 * Constructeur (par défaut)
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Jamais
 */

TInformationVoyageur::TInformationVoyageur() : fini(false)
{
    hautParleur = new HautParleur;

    #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif

    diffusionMessages.moveToThread(&threadDiffusionMessages);
    QObject::connect(this, SIGNAL(actualiserInformationsVoyageur(QString,QString,QString,QString,int)), &diffusionMessages, SLOT(actualiserInformationsVoyageur(QString,QString,QString,QString,int)), Qt::DirectConnection);
    QObject::connect(&threadDiffusionMessages, SIGNAL(started()), &diffusionMessages, SLOT(main()));
    QObject::connect(&threadDiffusionMessages, SIGNAL(finished()), &diffusionMessages, SLOT(terminer()));    

    numeroArret = -1;
    nombreArrets = 0;
    enCourse = PAS_EN_COURSE;
    periode = 0.;
    rayon   = 0.025; // 20 m
    pret    = false;
    arret   = false;
    fini    = false;
    bdd = BaseDeDonnees::getInstance();
    bdd->connecter("referentiel-123.sqlite");
}

/*
 * Destructeur
 *
 * Paramètre(s) en entrée : Jamais
 *
 * Paramètre(s) en sortie : Jamais
 *
 * Valeur de retour : Jamais
 */

TInformationVoyageur::~TInformationVoyageur()
{
    delete hautParleur;
    BaseDeDonnees::detruireInstance();
    #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif
}

// Le corps du thread -------------------------------------------------------------------------------------

void TInformationVoyageur::main()
{

    int enCourse = PAS_EN_COURSE;

    #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif

    threadDiffusionMessages.start();

    lireParametres();

    while(!fini)
    {
        #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO << " estPret() =" << estPret();
        #endif
        if(estPret())
        {
            enCourse = estEnCourse();
            #ifdef DEBUG_PANNEAU
            qDebug() << Q_FUNC_INFO << " enCourse() =" << enCourse;
            #endif
            if (enCourse == EN_COURSE)
            {
                detecterArret();
            }
            else if (enCourse == COURSE_TERMINEE)
            {
                finirCourse();
            }
        }
        attendrePeriode();
    }    

    diffusionMessages.finir();
    threadDiffusionMessages.quit();
    threadDiffusionMessages.wait();
}

// --------------------------------------------------------------------------------------------------------

/*
 * Rôle de la méthode : Attendre que le Thread TInformationVoyageur soit terminé
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune
 */

void TInformationVoyageur::finir()
{
    fini = true;
    annuler();
}

/*
 * Rôle de la méthode :
 *
 * Paramètre(s) en entrée :
 *
 * Paramètre(s) en sortie :
 *
 * Valeur de retour :
 */

void TInformationVoyageur::terminer()
{
    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << QThread::currentThreadId() << this;
    #endif
}

void TInformationVoyageur::lireParametres()
{
    // Le nom du fichier INI : nom-executable.ini
    QString fichierINI = qApp->applicationName() + ".ini"; // siv.ini

    QSettings parametres(fichierINI, QSettings::IniFormat);

    // periode course terminée
    periode = parametres.value("informationsVoyageur/periode", "10000").toInt();

    // Affichage de débugage
    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << QString::fromUtf8("période : %1").arg(periode);
    #endif
}

/*
 * Rôle du slot : Actualise les coordonnées GPS du BUS
 *
 * Paramètre(s) en entrée :
 *
 *  - Latitude : de type QString
 *  - Longitude : de type QString
 *
 * Paramètre(s) en sortie :
 *
 * Valeur de retour :
 */

void TInformationVoyageur::actualiserLocalisation(QString latitude, QString longitude)
{
    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << latitude << "," << longitude;
    #endif

    // Mise à jour de la localisation du bus

    latitudeBus = latitude;
    longitudeBus = longitude;
}

// Scénario "être informé des informations voyageurs" ---------------------------------------------------

/*
 * Rôle du slot :
 *
 * Paramètre(s) en entrée : numItinéraire
 *
 * Paramètre(s) en sortie :
 *
 * Valeur de retour : Aucune (méthode void)
 */

void TInformationVoyageur::actualiserCourse(QString numItineraire)
{
    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << numItineraire;
    #endif

    chargerItineraire(numItineraire);
}

/*
 * Rôle de la méthode :
 *
 * Paramètre(s) en entrée :
 *
 *  - numItineraire : de type QString
 *
 * Paramètre(s) en sortie :
 *
 * Valeur de retour : Aucune (méthode void)
 */

void TInformationVoyageur::chargerItineraire(QString numItineraire)
{
    itineraire.clear();

    // requete SQL
    QString requete = "SELECT itineraire.idLigne, nomCourt, destination, heureDepart, numeroSequence, nomLieu, latitude, longitude FROM itineraire INNER JOIN ligne ON ligne.idLigne=itineraire.idLigne INNER JOIN arret ON arret.idItineraire=itineraire.idItineraire INNER JOIN lieu ON lieu.idArret=arret.idArret WHERE itineraire.idItineraire='" + numItineraire + "' ORDER BY arret.numeroSequence ASC";
    bdd->recuperer(requete, itineraire);    

    if(!itineraire.isEmpty())
    {
        pret = true;

        nombreArrets = itineraire.size();
        numeroArret = 0;

        #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO << "ITINERAIRE CHARGE !" << itineraire << "TAILLE" << nombreArrets;
        #endif
    }
    else
    {
        pret = false;
        #ifdef DEBUG_PANNEAU
        qDebug() << Q_FUNC_INFO << "Aucun itineraire n'a été chargé !";
        #endif
    }
}

/*
 * Rôle de la méthode :
 *
 * Paramètre(s) en entrée :
 *
 * Paramètre(s) en sortie :
 *
 * Valeur de retour :
 */

bool TInformationVoyageur::estPret()
{
    // Un itineraire a été chargé

    return pret;
}

/*
 * Rôle de la méthode : Savoir si le bus est sur le parcours
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie :
 *
 * Valeur de retour : Un entier
 */

int TInformationVoyageur::estEnCourse()
{
    // Si le bus est sur le parcours

    if(numeroArret == -1)
    {
        enCourse = PAS_EN_COURSE;
    }
    else if(numeroArret < nombreArrets)
    {
        enCourse = EN_COURSE;
    }

    return enCourse;
}

/*
 * Rôle de la méthode :
 *
 * Paramètre(s) en entrée :
 *
 * Paramètre(s) en sortie :
 *
 * Valeur de retour :
 */

bool TInformationVoyageur::finirCourse()
{
    // Terminus
    // emit terminerCourse();

    return false;
}

/*
 * Rôle de la méthode :
 *
 * Paramètre(s) en entrée :
 *
 * Paramètre(s) en sortie :
 *
 * Valeur de retour :
 */

void TInformationVoyageur::attendrePeriode()
{
    //attente prochaine diffusion
    this->msleep(periode);
}

// Scénario "detecter les points d'arrêts" ---------------------------------------------------------------

/*
 * Rôle de la méthode : Savoir lorsque le bus entre et sort d'une zone d'arrêt
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (méthode void)
 */

void TInformationVoyageur::detecterArret()
{
    bool debut = detecterDebutArret();
    bool fin = detecterFinArret();

    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << "Numero arret : " << numeroArret;
    #endif

    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << "Debut : " << debut;
    #endif

    if (debut == true)
    {        
        declencherDebutArret();
    }

    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << "Fin : " << fin;
    #endif

    if (fin == true)
    {
        declencherFinArret();
    }
}

/*
 * Rôle de la méthode : Lorsque le bus entre dans la zone d'arrêt
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Renvoi vrai si le bus entre dans dans la zone d'arrêt, sinon faux
 */

bool TInformationVoyageur::detecterDebutArret()
{       
    if(latitudeBus.isEmpty() || longitudeBus.isEmpty())
    {
        return false;
    }

    QString latitudeArret, longitudeArret;

    latitudeArret = itineraire.at(numeroArret).at(6);
    longitudeArret = itineraire.at(numeroArret).at(7);    

    double distance = calculerDistance(latitudeBus, longitudeBus, latitudeArret, longitudeArret);

    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << "Latitude/Longitude BUS : " << latitudeBus << longitudeBus;
    qDebug() << Q_FUNC_INFO << "Latitude/Longitude ARRET : " << latitudeArret << longitudeArret;
    qDebug() << Q_FUNC_INFO << "Distance : " << distance << "Rayon : " << rayon;
    #endif

    if (distance < rayon)
    {
        return true;
    }
    else
    {
        return false;
    }
}

/*
 * Rôle de la méthode : Retourne la distance entre le bus et la zone d'arrêt
 *
 * Paramètre(s) en entrée :
 *      - latitudeBus : la latitude du bus reçue par le gps
 *      - longitudeBus : la longitude du bus reçue par le gps
 *      - latitudeZoneArret : la latitude fixe de la zone d'Arret
 *      - longitudeZoneArret : la longitude fixe de la zone d'Arret
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : la distance sous forme d'un double
 */

double TInformationVoyageur::calculerDistance(QString latitudeBus, QString longitudeBus, QString latitudeArret, QString longitudeArret)
{
    double rayonTerreRadians = 6378137;   // Terre = sphère de 6378km de rayon
    double distanceLatitude = 0.;
    double distanceLongitude = 0.;
    double distance = 0.;

    double latitudeBusDouble = 0.;
    double longitudeBusDouble = 0.;
    double latitudeArretDouble = 0.;
    double longitudeArretDouble = 0.;

    double latitudeBusDoubleRadians = 0.;
    double longitudeBusDoubleRadians = 0.;
    double latitudeArretDoubleRadians = 0.;
    double longitudeArretDoubleRadians = 0.;

    // Conversion des QString en double (degrés)

    latitudeBusDouble = latitudeBus.toDouble();
    longitudeBusDouble = longitudeBus.toDouble();
    latitudeArretDouble = latitudeArret.toDouble();
    longitudeArretDouble = longitudeArret.toDouble();

    // Conversion Degrés > Radians

    latitudeBusDoubleRadians = (latitudeBusDouble*(PI/180));
    longitudeBusDoubleRadians = (longitudeBusDouble*(PI/180));
    latitudeArretDoubleRadians = (latitudeArretDouble*(PI/180));
    longitudeArretDoubleRadians = (longitudeArretDouble*(PI/180));

    // Formules de calcul avec les coordonnées du bus et de la zone d'Arret (latitude et longitude)

    distanceLatitude = (latitudeArretDoubleRadians - latitudeBusDoubleRadians)/2;
    distanceLongitude = (longitudeArretDoubleRadians - longitudeBusDoubleRadians)/2;
    double a = (sin(distanceLatitude)*sin(distanceLatitude))+(cos(latitudeBusDoubleRadians)*cos(latitudeArretDoubleRadians)*sin(distanceLongitude)*sin(distanceLongitude));
    distance = 2*atan2(sqrt(a), sqrt(1-a));

    return ((rayonTerreRadians*distance)/1000);
}

/*
 * Rôle de la méthode : Lorsque le bus est entré dans la zone d'arrêt
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (méthode void)
 */

void TInformationVoyageur::declencherDebutArret()
{
    setZoneArret(true);
    signalerArret();

    // TODO : cas du terminus
}

/*
 * Rôle de la méthode : Fixer si le bus est dans la zone d'arrêt ou non
 *
 * Paramètre(s) en entrée : etat bool (vrai si le bus est dans la zone d'arrêt)
 *
 * Paramètre(s) en sortie : attribut arret modifié
 *
 * Valeur de retour : aucun
 */

void TInformationVoyageur::setZoneArret(bool etat)
{
    arret = etat;
}

/*
 * Rôle de la méthode : Signaler aux Voyageurs l'arret en cours et le prochain arrêt en fonction de la position GPS
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Aucune (méthode void)
 */

void TInformationVoyageur::signalerArret()
{
    QString numLigne, destination, arretEnCours, prochainArret;

    numLigne = itineraire.at(0).at(1);
    destination = itineraire.at(0).at(2);
    arretEnCours = itineraire.at(numeroArret).at(5);
    if(numeroArret < (nombreArrets-1))
    {
        prochainArret = itineraire.at(numeroArret+1).at(5);
    }
    else
    {
        prochainArret = ""; // pas de prochain arrêt
    }

    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << "numLigne : " << numLigne << "destination : " << destination << "arretEnCours : " << arretEnCours << "prochainArret : " << prochainArret;
    #endif

    emit actualiserInformationsVoyageur(numLigne, destination, arretEnCours, prochainArret, numeroArret);

    if (arret == true)
    {

        hautParleur->parle(arretEnCours);
    }
    else
    {
        if(!prochainArret.isEmpty())
        {

            hautParleur->parle(prochainArret);
        }
    }
}

/*
 * Rôle de la méthode : Lorsque le bus sort de la zone d'arrêt
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : Aucun
 *
 * Valeur de retour : Renvoi vrai si le bus sort de la zone d'arrêt, sinon faux
 */

bool TInformationVoyageur::detecterFinArret()
{
    if(latitudeBus.isEmpty() || longitudeBus.isEmpty())
        return false;

    QString latitudeArret, longitudeArret;

    latitudeArret = itineraire.at(numeroArret).at(6);
    longitudeArret = itineraire.at(numeroArret).at(7);

    double distance = calculerDistance(latitudeBus, longitudeBus, latitudeArret, longitudeArret);

    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << "Latitude/Longitude BUS : " << latitudeBus << longitudeBus;
    qDebug() << Q_FUNC_INFO << "Latitude/Longitude ARRET : " << latitudeArret << longitudeArret;
    qDebug() << Q_FUNC_INFO << "Distance : " << distance << "Rayon : " << rayon;
    #endif

    if (distance > rayon)
    {
        return true;
    }
    else
    {
        return false;
    }
}

/*
 * Rôle de la méthode : Lorsque le bus est sorti de la zone d'arrêt
 *
 * Paramètre(s) en entrée : Aucun
 *
 * Paramètre(s) en sortie : setZoneArret() faux si le bus n'est pas entré dans la Zone d'Arret
 *
 * Valeur de retour : Aucune (méthode void)
 */

void TInformationVoyageur::declencherFinArret()
{
    #ifdef DEBUG_PANNEAU
    qDebug() << Q_FUNC_INFO << "Arret : " << arret;
    #endif

    if (arret == true)
    {
        setZoneArret(false);
        signalerArret();        
        if(numeroArret < (nombreArrets-1))
        {
            numeroArret++; //on passe au prochain arret
        }
        else
        {
            // la course est terminée
            enCourse = COURSE_TERMINEE;
        }
    }
}
