#include "dialogue.h"
#include <QDebug>

Dialogue::Dialogue( QObject *parent ) : QObject( parent )
{
   connecte = false;   
}

Dialogue::~Dialogue()
{
   delete serveur;
}

void Dialogue::demarrer(QString adresseIP, int port)
{    
   if(connecte == true)
      return;

   // une socket à connecter au serveur SAI
   serveur = new QTcpSocket;

   QHostAddress *adresseServeur = new QHostAddress(adresseIP);
   serveur->connectToHost(adresseServeur->toString(), port);
   if(!serveur->isOpen())
   {
      qDebug() << Q_FUNC_INFO << QString::fromUtf8("erreur ouverture connexion :") << serveur->errorString();
      return;
   }
   else
   {
      connect(serveur, SIGNAL(connected()), this, SLOT(estConnecte()));
      connect(serveur, SIGNAL(disconnected()), this, SLOT(estDeconnecte()));
      connect(serveur, SIGNAL(readyRead()), this, SLOT(recevoir()));

      qDebug() << Q_FUNC_INFO << QString::fromUtf8("connexion en cours");
      if(!serveur->waitForConnected(1000))
      {
         qDebug() << Q_FUNC_INFO << QString::fromUtf8("erreur connexion :") << serveur->errorString();
         return;
      }

      // bascule l'objet socket dans le thread
      serveur->moveToThread(this->thread());
   }
}

void Dialogue::arreter()
{
   // on se déconnecte du serveur
   serveur->disconnectFromHost();
}

void Dialogue::estConnecte()
{
   qDebug() << Q_FUNC_INFO << QString::fromUtf8("connexion réussie au serveur :") << serveur->peerAddress().toString() << QString::number(serveur->peerPort());
   connecte = true;
}

void Dialogue::estDeconnecte()
{
   qDebug() << Q_FUNC_INFO << QString::fromUtf8("déconnexion réussie du serveur :") << serveur->peerAddress().toString() << QString::number(serveur->peerPort());
   connecte = false;
}

void Dialogue::recevoir()
{
   QByteArray donnees;
   donnees = serveur->readAll();

   //qDebug() << Q_FUNC_INFO << QString::fromUtf8("réception du serveur :") << serveur->peerAddress().toString() << serveur->peerPort();
   QString message(donnees.constData());

   emit donneesRecues(message);
}

bool Dialogue::envoyer(const QString &message)
{
   //qDebug() << Q_FUNC_INFO << QString::fromUtf8("-> serveur :") << message << connecte;

   if(connecte == false)
      return false;

   qint64 nb = serveur->write(qPrintable(message));

   //qDebug() << Q_FUNC_INFO << QString::fromUtf8("-> serveur :") << nb;
   if(nb > 0)
       return true;

   return false;
}
