#include <QtGui>

#include "ihm.h"
#include "simulateurbus.h"
#include "simulation.h"

IHM::IHM( QWidget *parent ) : QWidget( parent )
{
   labelAdresseServeur = new QLabel(this);
   labelAdresseServeur->setText(QString::fromUtf8("Adresse IP SAI : "));
   leAdresseServeur = new QLineEdit(this);
   leAdresseServeur->setText("192.168.52.2");
   labelPortServeur = new QLabel(this);
   labelPortServeur->setText(QString::fromUtf8("Port IP SAI : "));
   lePortServeur = new QLineEdit(this);
   lePortServeur->setText("5000");
   labelIdBus = new QLabel(this);
   labelIdBus->setText(QString::fromUtf8("Id bus : "));
   leIdBus = new QLineEdit(this);
   leIdBus->setText("155");
   labelIdService = new QLabel(this);
   labelIdService->setText(QString::fromUtf8("Id service : "));
   leIdService = new QLineEdit(this);
   leIdService->setText("1");
   labelLigne = new QLabel(this);   
   labelLigne->setText(QString::fromUtf8("Ligne : "));
   choixLigne = new QComboBox(this);   
   labelItineraire = new QLabel(this);   
   labelItineraire->setText(QString::fromUtf8("Itinéraire : "));
   choixItineraire = new QComboBox(this);   
   choixItineraire->setEnabled(false);
   vueTrajet = new QTableView(this);

   messagesSIV = new QTextEdit(this);
   messagesSIV->setReadOnly(true);
  
   bDemarrer  = new QPushButton(QString::fromUtf8("Démarrer"), this);  
   bDemarrer->setEnabled(false);
   bArreter   = new QPushButton(QString::fromUtf8("Arrêter"), this);  
   bArreter->setEnabled(false);
   bQuitter   = new QPushButton("Quitter", this);    

   lireParametresINI();

   QHBoxLayout *hLayout0a = new QHBoxLayout;
   QHBoxLayout *hLayout0b = new QHBoxLayout;
   QHBoxLayout *hLayout0 = new QHBoxLayout;
   QHBoxLayout *hLayout1 = new QHBoxLayout;
   QHBoxLayout *hLayout2 = new QHBoxLayout;
   QHBoxLayout *hLayout3 = new QHBoxLayout;
   QHBoxLayout *hLayout4 = new QHBoxLayout;
   QVBoxLayout *mainLayout = new QVBoxLayout;   
   hLayout0a->addWidget(labelAdresseServeur);
   hLayout0a->addWidget(leAdresseServeur);
   hLayout0a->addWidget(labelPortServeur);
   hLayout0a->addWidget(lePortServeur);
   hLayout0b->addWidget(labelIdBus);
   hLayout0b->addWidget(leIdBus);
   hLayout0b->addWidget(labelIdService);
   hLayout0b->addWidget(leIdService);
   hLayout0->addWidget(labelLigne);
   hLayout0->addWidget(choixLigne);
   hLayout1->addWidget(labelItineraire);
   hLayout1->addWidget(choixItineraire);
   hLayout2->addWidget(vueTrajet);
   hLayout3->addWidget(messagesSIV);
   hLayout4->addWidget(bDemarrer);
   hLayout4->addWidget(bArreter);
   hLayout4->addWidget(bQuitter);
   mainLayout->addLayout(hLayout0a);
   mainLayout->addLayout(hLayout0b);
   mainLayout->addLayout(hLayout0);
   mainLayout->addLayout(hLayout1);
   mainLayout->addLayout(hLayout2);
   mainLayout->addLayout(hLayout3);
   mainLayout->addLayout(hLayout4);
   setLayout(mainLayout);

   initialiserBD();

   qRegisterMetaType <SimulateurBus * >("SimulateurBus *");

   connect(choixLigne, SIGNAL(currentIndexChanged(int)), this, SLOT(selectionnerLigne(int)));  
   connect(choixItineraire, SIGNAL(currentIndexChanged(int)), this, SLOT(selectionnerItineraire(int)));  
   connect(bDemarrer, SIGNAL(clicked()), this, SLOT(demarrer()));
   connect(bArreter, SIGNAL(clicked()), this, SLOT(arreter()));
   connect(bQuitter, SIGNAL(clicked()), this, SLOT(quitter()));
   connect(this, SIGNAL(quit()), qApp, SLOT(quit()));

   setWindowTitle(QString::fromUtf8("Simulateur Flotte Bus"));
   setFixedHeight(sizeHint().height());
   move(0, 0);
}

void IHM::demarrer() 
{  
    for (int i = 0; i < simulateursBus.size(); ++i)
    {
        if (simulateursBus.at(i)->getIdBus() == leIdBus->text().toInt())
        {
             QMessageBox::critical(0, QObject::tr("Erreur !"), QObject::tr("Impossible de simuler ce bus !\n%1").arg("Il est déjà en simulation !"));
             simulateursBus.at(i)->show();
             simulateursBus.at(i)->raise();
             simulateursBus.at(i)->activateWindow();
             return;
        }
    }
    SimulateurBus* simulateurBus = new SimulateurBus(leIdBus->text().toInt(), leIdService->text().toInt(), db);
    simulateursBus.push_back(simulateurBus);

    connect(simulateurBus, SIGNAL(signalerMessage(QString)), this, SLOT(actualiserMessage(QString)));
    connect(this, SIGNAL(terminer()), simulateurBus, SLOT(quitter()));
    connect(simulateurBus, SIGNAL(fin(SimulateurBus *)), this, SLOT(finirSimulation(SimulateurBus *)));

    simulateurBus->setTrajet(trajet, route_id, trip_id);
    simulateurBus->setServeurSAI(leAdresseServeur->text(), lePortServeur->text().toInt());
    simulateurBus->setLigne(choixLigne->currentText(), choixItineraire->currentText());

    simulateurBus->show();
    simulateurBus->raise();
    simulateurBus->activateWindow();

    choixLigne->setCurrentIndex(0);
    itineraires.clear();
    choixItineraire->clear();
    choixItineraire->addItem("");
    choixItineraire->setEnabled(false);
    vueTrajet->setModel(NULL);
    bDemarrer->setEnabled(false);
    bArreter->setEnabled(true);
    leIdBus->setFocus();
}

IHM::~IHM()
{
    ecrireParametresINI();
}

void IHM::arreter() 
{      
    emit terminer();
}

void IHM::quitter() 
{  
   arreter();

   emit quit();
}

void IHM::selectionnerLigne(int index) 
{
   if(index > 0)
   {
      //qDebug() << "ligne choisie :" << lignes.at(index-1);
      QString route_short_name = lignes.at(index-1).at(0);
      
      bool retour;
      QSqlQuery query;
      QString requete = "SELECT routes.route_id, routes.route_short_name, routes.route_long_name, trips.trip_id, trips.trip_headsign, trips.direction_id, stop_times.stop_id, stop_times.stop_sequence, stop_times.departure_time, stops.stop_name, stops.stop_lat, stops.stop_lon FROM routes INNER JOIN trips ON routes.route_id=trips.route_id INNER JOIN stop_times ON stop_times.trip_id=trips.trip_id INNER JOIN stops ON stops.stop_id=stop_times.stop_id WHERE route_short_name=\'" + route_short_name + "\' AND stop_times.stop_sequence=\'0\' ORDER BY departure_time ASC";
      retour = query.exec(requete);
      if(retour)
      {
         itineraires.clear();
         choixItineraire->clear();
         choixItineraire->addItem("");   
         while ( query.next() )  
         {  
            //qDebug() << "enregistrement -> ";
            //for(int i=0;i<query.record().count();i++)
               //qDebug() << query.value(i).toString();
            QStringList itineraire;
            QString route_id = query.value(0).toString();
            itineraire << route_id;			
            QString trip_id = query.value(3).toString();
            itineraire << trip_id;			
            QString trip_headsign = query.value(4).toString();
            itineraire << trip_headsign;			
            QString departure_time = query.value(8).toString();
            itineraire << departure_time;			
            QString stop_name = query.value(9).toString();
            itineraire << stop_name;			
            itineraires.push_back(itineraire);
            
            choixItineraire->addItem(departure_time + " " + stop_name + " -> " + trip_headsign);   
         } 
         
         choixItineraire->setEnabled(true);
      }
      else  qDebug() << query.lastError().text();
   }
   else
   {
      itineraires.clear();
      choixItineraire->clear();
      choixItineraire->addItem("");   
      choixItineraire->setEnabled(false);
   }
}

void IHM::selectionnerItineraire(int index) 
{
   if(index > 0)
   {
      //qDebug() << "itineraire choisi :" << itineraires.at(index-1);
      route_id = itineraires.at(index-1).at(0);
      trip_id = itineraires.at(index-1).at(1);
      
      bool retour;
      QSqlQuery query;
      //QString requete = "SELECT trips.trip_headsign, trips.direction_id,  stop_times.stop_id, stop_times.stop_sequence, stop_times.arrival_time, stop_times.departure_time, stops.stop_name, stops.stop_lat, stops.stop_lon FROM trips INNER JOIN stop_times ON stop_times.trip_id=trips.trip_id INNER JOIN stops ON stops.stop_id=stop_times.stop_id WHERE trips.trip_id=\'" + trip_id + "\' ORDER BY departure_time ASC";
      QString requete = "SELECT trips.trip_headsign, trips.direction_id, stop_times.stop_id, stop_times.stop_sequence, stop_times.arrival_time, stop_times.departure_time, stops.stop_name, stops.stop_lat, stops.stop_lon, trips.shape_id FROM trips INNER JOIN stop_times ON stop_times.trip_id=trips.trip_id INNER JOIN stops ON stops.stop_id=stop_times.stop_id WHERE trips.trip_id=\'" + trip_id + "\' ORDER BY stop_sequence ASC";
      //qDebug() << requete;
      retour = query.exec(requete);
      if(retour)
      {
         trajet.clear();
         while ( query.next() )  
         {  
            QStringList arret;
            for(int i=0;i<query.record().count();i++)
               arret << query.value(i).toString();         
            trajet.push_back(arret);
         } 
         //qDebug() << "Trajet : " << trajet;
      }
      else  qDebug() << query.lastError().text();
     
      QSqlQueryModel *model = new QSqlQueryModel;
      model->setQuery("SELECT stops.stop_name, stop_times.arrival_time, stop_times.departure_time FROM trips INNER JOIN stop_times ON stop_times.trip_id=trips.trip_id INNER JOIN stops ON stops.stop_id=stop_times.stop_id WHERE trips.trip_id=\'" + trip_id + "\' ORDER BY stop_sequence ASC");
      QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
      model->setHeaderData(0, Qt::Horizontal, "Arrêt");
      model->setHeaderData(1, Qt::Horizontal, "Arrivée");
      model->setHeaderData(2, Qt::Horizontal, "Départ");
      vueTrajet->setModel(model);
      vueTrajet->resizeColumnsToContents();
      //vueTrajet->show();
      bArreter->setEnabled(false);
      bDemarrer->setEnabled(true);
   }
}

void IHM::actualiserMessage(const QString &message)
{
   messagesSIV->append(message);
}

void IHM::lireParametresINI()
{
    QString fichierINI = qApp->applicationDirPath() + "/" + qApp->applicationName() + ".ini";
    QSettings parametres(fichierINI, QSettings::IniFormat);
    parametres.setIniCodec(QTextCodec::codecForName("UTF-8"));

    QString adresseIP = parametres.value("sai/ip", "127.0.0.1").toString();
    int port = parametres.value("sai/port", "5000").toInt();

    leAdresseServeur->setText(adresseIP);
    lePortServeur->setText(QString::number(port));
}

void IHM::ecrireParametresINI()
{
    QString fichierINI = qApp->applicationDirPath() + "/" + qApp->applicationName() + ".ini";
    QSettings parametres(fichierINI, QSettings::IniFormat);
    parametres.setIniCodec(QTextCodec::codecForName("UTF-8"));

    parametres.beginGroup("sai");
    parametres.setValue("ip", leAdresseServeur->text());
    parametres.setValue("port", lePortServeur->text());
    parametres.endGroup();
}

void IHM::initialiserBD()
{
   db = QSqlDatabase::addDatabase("QSQLITE");   
   
   db.setDatabaseName(QCoreApplication::applicationDirPath() + "/referentiel/referentiel-tisseo.sqlite");
   if(!db.open())
   {
      //qDebug() << "referentiel.sqlite erreur !";
      return ;
   }

   QSqlQuery query("SELECT route_short_name,route_long_name FROM routes");
   query.next(); // nom des colonnes
   choixLigne->addItem("");   
   while (query.next()) 
   {
      QStringList route;
      QString route_short_name = query.value(0).toString();
      route << route_short_name;			
      QString route_long_name = query.value(1).toString();
      route << route_long_name;			
      lignes.push_back(route);
      
      //qDebug() << route_short_name << " - " << route_long_name;
      choixLigne->addItem(route_short_name + " - " + route_long_name);
   }
}

void IHM::finirSimulation(SimulateurBus *simulateurBus)
{
    for (int i = 0; i < simulateursBus.size(); ++i)
    {
        if (simulateursBus.at(i) == simulateurBus)
        {
            simulateursBus.removeAt(i);
            break;
        }
    }
    if(simulateursBus.size() > 0) bArreter->setEnabled(true);
    else                          bArreter->setEnabled(false);
}
