#include "simulation.h"
#include "geolocalisation.h"
#include "dialogue.h"

Simulation::Simulation()
{
   dialogue = new Dialogue(this);
   stopped = false;
   stopSequence = 0;
   stopId = 0;
   tempsArret = 0;
   tempsDeplacement = 0;
   delimiteur = ";";
   duree = 0;
}

Simulation::~Simulation()
{
}

void Simulation::run()
{
   this->setPriority(QThread::NormalPriority);
   
   qDebug() << Q_FUNC_INFO << "demarrage ...\n";
   try
   {
      dialogue->demarrer(adresseIP, port);
      connect(dialogue, SIGNAL(donneesRecues(const QString &)), this, SLOT(receptionnerMessage(const QString &)));

      distanceParcourue = 0.;
      topDepart = QTime::currentTime();
      QString heureDepart = trajet.at(0).at(DEPARTURE_TIME);
      topDepartSimule = QTime::fromString(heureDepart, "hh:mm:ss");

      while(!isStopped())
      {                      
         simulerTrajet();
         
         // prochain arrêt
         QThread::usleep(tempsArret);
         duree += tempsArret/1000000.;
      }
   }
   catch (...)
   {
      // faire quelque chose avec les exceptions
   }
   QMutexLocker verrou_stop(&mutex);
   stopped = false;

   dialogue->arreter();
   disconnect(dialogue, SIGNAL(donneesRecues(const QString &)), this, SLOT(receptionnerMessage(const QString &)));
   
   qDebug() << Q_FUNC_INFO << "terminee.\n";
}

void Simulation::stop()
{
    QMutexLocker verrou_stop(&mutex);
    stopped = true;
}

bool Simulation::isStopped()
{
    QMutexLocker verrou_stop(&mutex);
    return stopped;
}

int Simulation::getIdBus()
{
    return this->idBus;
}

void Simulation::setRayon(double rayon)
{
    this->rayon = rayon;
}

void Simulation::setTemps(unsigned long tempsArret, unsigned long tempsDeplacement)
{
    // milli s -> micro s
    this->tempsArret = tempsArret * 1000;
    this->tempsDeplacement = tempsDeplacement * 1000;
}

void Simulation::setTrajet(const QVector<QStringList> &trajet, const QVector<QStringList> &trajetDetails)
{
   this->trajet = trajet;
   this->trajetDetails = trajetDetails;
}

void Simulation::setServeurSAI(const QString &adresseIP, int port)
{
    this->adresseIP = adresseIP;
    this->port = port;
}

void Simulation::setId(int idBus, int idService, QString route_id, QString trip_id)
{
    this->idBus = idBus;
    this->idService = idService;
    this->route_id = route_id;
    this->trip_id = trip_id;
}

void Simulation::setAlerte(int codeAlerte)
{
    QString phrase;
    QString type = "ALERTE";
    QString messageAlerte;
    QTime maintenant;
    QDate aujourdhui;

    switch(codeAlerte)
    {
    case 1:
        messageAlerte = (QString::fromUtf8("Déviation"));
        break;
    case 2:
        messageAlerte = (QString::fromUtf8("Bouchon"));
        break;
    case 3:
        messageAlerte = (QString::fromUtf8("Billetique"));
        break;
    case 4:
        messageAlerte = (QString::fromUtf8("Bus"));
        break;
    case 5:
        messageAlerte = (QString::fromUtf8("Agression"));
        break;
    default:
        messageAlerte = (QString::fromUtf8("Autre"));
        break;
    }

    phrase = getTrameId(type);

    phrase += QString::number(400 + codeAlerte) + delimiteur + messageAlerte + delimiteur;

    unsigned char checksum = calculerChecksum(phrase);
    QString strChecksum = QString("%1").arg(checksum, 2, 16, QLatin1Char('0'));
    phrase += strChecksum.toUpper();
    phrase += QString("\r\n");

    dialogue->envoyer(phrase);

    phrase.remove(phrase.size()-2, 2); // enlève le \r\n pour l'ihm

    QString trame = "<b><font color=red>" + phrase + "</font></b>";
    emit nouvelleTrame(trame);
}

void Simulation::simulerTrajet()
{
    QString arret;
    QString latitude, longitude;
    QString latitudeS, longitudeS;
    QString latitudeA, longitudeA;
    int id;
    double cap, vitesse;

    //qDebug() << Q_FUNC_INFO << trajet;
    for (int i = stopId; i < trajetDetails.size(); ++i)
    {
        latitude = trajetDetails.at(i).at(1);
        longitude = trajetDetails.at(i).at(2);
        id = getArret(latitude, longitude);

        if((i + 1) < trajetDetails.size())
        {
            latitudeS = trajetDetails.at(i+1).at(1);
            longitudeS = trajetDetails.at(i+1).at(2);
            cap = getCap(latitude, longitude, latitudeS, longitudeS);
        }
        else
            cap = 0.;

        if(i != 0)
        {
            latitudeA = trajetDetails.at(i-1).at(1);
            longitudeA = trajetDetails.at(i-1).at(2);
            vitesse = getVitesse(latitudeA, longitudeA, latitude, longitude);
        }
        else
            vitesse = 0.;

        emettreTrameLocalisation(latitude, longitude, QString::number(cap), QString::number(vitesse));

        if(id != -1)
        {
            //qDebug() << Q_FUNC_INFO << latitude << longitude;
            //if(trajet.at(id).at(STOP_SEQUENCE) == QString::number(stopSequence))
            {
				if(id < trajet.size())
				{
					arret = trajet.at(id).at(STOP_NAME);

					emettreTrameArret(arret);

					QTime maintenantSimule;
					maintenantSimule = topDepartSimule.addSecs((int)duree*10);
					emit nouvelArretHorodate(arret, maintenantSimule.toString("hh:mm:ss"));
					//emit nouvelArret(arret);
					stopSequence++;
				}
                stopId = i;

                return;
            }
        }

        // prochaine localisation
        QThread::usleep(tempsDeplacement);
        duree += tempsDeplacement/1000000.;
    }

    qDebug() << Q_FUNC_INFO << "trajet fini";
    emit trajetFini();
    stopped = true;
}

QString Simulation::getTrameId(QString type)
{
   QString entete;
   QTime maintenant;
   QDate aujourdhui;
   QTime maintenantSimule;

   maintenantSimule = topDepartSimule.addSecs((int)duree*10);

   //entete = type + delimiteur + aujourdhui.currentDate().toString("dd/MM/yyyy") + delimiteur + maintenant.currentTime().toString("hh:mm:ss.zzz") + delimiteur;
   entete = type + delimiteur + aujourdhui.currentDate().toString("dd/MM/yyyy") + delimiteur + maintenantSimule.toString("hh:mm:ss.zzz") + delimiteur;
   entete += QString::number(idBus) + delimiteur + QString::number(idService) + delimiteur + route_id  + delimiteur + trip_id + delimiteur;

   //qDebug() << Q_FUNC_INFO << "entete = " << entete;
   emit nouvelHorodatage(maintenantSimule.toString("hh:mm:ss"));

   return entete;
}

void Simulation::emettreTrameLocalisation(QString latitude, QString longitude, QString cap, QString vitesse)
{
    QString phrase;
    QString type = "LOCALISATION";
    bool etat;
    int nbTentatives = 0;

    phrase = getTrameId(type);

    phrase += latitude + delimiteur + "N" + delimiteur + longitude + delimiteur + "E" + delimiteur;
    phrase += cap + delimiteur + vitesse + delimiteur;

    unsigned char checksum = calculerChecksum(phrase);
    QString strChecksum = QString("%1").arg(checksum, 2, 16, QLatin1Char('0'));
    phrase += strChecksum.toUpper();
    phrase += QString("\r\n");

    do
    {
        etat = dialogue->envoyer(phrase);
        if(etat == false)
        {
            nbTentatives++;
            msleep(500);
        }
    }
    while (etat != true && nbTentatives != 3);

    phrase.remove(phrase.size()-2, 2); // enlève le \r\n pour l'ihm
    QString trame = "<b><font color=green>" + phrase + "</font></b>";
    emit nouvelleTrame(trame);
}

void Simulation::emettreTrameArret(QString arret)
{
    QString phrase;
    QString type = "ARRET";
    bool etat;
    int nbTentatives = 0;

    phrase = getTrameId(type);

    phrase += arret + delimiteur;

    unsigned char checksum = calculerChecksum(phrase);
    QString strChecksum = QString("%1").arg(checksum, 2, 16, QLatin1Char('0'));
    phrase += strChecksum.toUpper();
    phrase += QString("\r\n");

    do
    {
        etat = dialogue->envoyer(phrase);
        if(etat == false)
        {
            nbTentatives++;
            msleep(500);
        }
    }
    while (etat != true && nbTentatives != 3);

    phrase.remove(phrase.size()-2, 2); // enlève le \r\n pour l'ihm
    QString trame = "<b><font color=blue>" + phrase + "</font></b>";
    emit nouvelleTrame(trame);
}

unsigned char Simulation::calculerChecksum(QString data)
{
   unsigned char checksum = 0;   
   
   for(int i=0;i<data.length();i++)
   {
      //printf("0x%02X ", data.at(i).toAscii());
      checksum ^= data.at(i).toAscii(); // XOR
   }
   //printf("\n");   
   //qDebug("(checksum) 0x%02X\n", checksum);
      
   return checksum;
}

void Simulation::receptionnerMessage(const QString &message)
{
    emit nouveauMessage(message);
}

int Simulation::getArret(QString latitude, QString longitude)
{
    int id = -1;
    QString latitudeA;
    QString longitudeA;
    double distance = 0.;
    double distanceMin = 10000;
    Geolocalisation *point1, *point2;

    point1 = new Geolocalisation(latitude.toDouble(), longitude.toDouble()*-1);
    //qDebug() << Q_FUNC_INFO << "*** recherche arret";
    int i = stopSequence;
    //for (int i = stopSequence; i < trajet.size(); ++i)
    {
		if(i < trajet.size())
		{
			latitudeA = trajet.at(i).at(STOP_LAT);
			longitudeA = trajet.at(i).at(STOP_LON);
			point2 = new Geolocalisation(latitudeA.toDouble(), longitudeA.toDouble()*-1);
			distance = point1->distance(*point2) * 1000;
			//qDebug() << Q_FUNC_INFO << "### distance =" << distance;
			if(distance < distanceMin)
				distanceMin = distance;
			if(distance < rayon)
			{
				//qDebug() << Q_FUNC_INFO << "### distance =" << distance;
				qDebug() << Q_FUNC_INFO << "### arret =" << trajet.at(i).at(STOP_NAME) << "- stop sequence =" << trajet.at(i).at(STOP_SEQUENCE) << "- distance =" << distance;
				return trajet.at(i).at(STOP_SEQUENCE).toInt();
			}
		}
    }
    //qDebug() << Q_FUNC_INFO << "*** distance min =" << distanceMin;
    //qDebug() << Q_FUNC_INFO << "*** arret non trouve";
    return id;
}

double Simulation::getCap(QString latitude, QString longitude, QString latitudeS, QString longitudeS)
{
    double orientation = 0.;
    Geolocalisation *point1, *point2;

    point1 = new Geolocalisation(latitude.toDouble(), longitude.toDouble()*-1);
    point2 = new Geolocalisation(latitudeS.toDouble(), longitudeS.toDouble()*-1);
    orientation = point1->orientation(*point2);

    //qDebug() << Q_FUNC_INFO << "orientation = " << orientation;

    return orientation;
}

double Simulation::getVitesse(QString latitudeA, QString longitudeA, QString latitude, QString longitude)
{
    double distance, vitesse = 0.;
    Geolocalisation *point1, *point2;
    QTime maintenantSimule;
    //QTime horodatage = QTime::currentTime();

    maintenantSimule = topDepartSimule.addSecs((int)duree);

    point1 = new Geolocalisation(latitudeA.toDouble(), longitudeA.toDouble()*-1);
    point2 = new Geolocalisation(latitude.toDouble(), longitude.toDouble()*-1);
    distance = point1->distance(*point2) * 1000;
    distanceParcourue += distance;
    //vitesse = (distanceParcourue / 1000.) / ((double)topDepart.secsTo(horodatage)*5. / 3600.);
    //vitesse = (distanceParcourue / 1000.) / ((double)topDepartSimule.secsTo(maintenantSimule) / 3600.);
    vitesse = (distance / 1000.) / ((double)topDepartSimule.secsTo(maintenantSimule) / 3600.);

    //qDebug() << Q_FUNC_INFO << "intervalle = " << topDepartSimule.secsTo(maintenantSimule);
    //qDebug() << Q_FUNC_INFO << "distance = " << distance;
    //qDebug() << Q_FUNC_INFO << "distance parcourue = " << distanceParcourue;
    //qDebug() << Q_FUNC_INFO << "vitesse = " << vitesse;
    //qDebug();

    emit nouvelleDistance(QString::number(distanceParcourue));
    emit nouvelleVitesse(QString::number(vitesse));

    return vitesse;
}
