#ifndef SIMULATION_H
#define SIMULATION_H

#include <QThread>
#include <QMutexLocker>
#include <QMutex>
#include <QtGui>

#include <iostream>
#include <iomanip>
#include <sstream> // stringstream !
#include <string>
#include <cstdio>
#include <stdint.h>

using namespace std;

//#define DEBUG_SIMULATION

#define TRIP_HEADSIGN   0
#define DIRECTION_ID    1
#define STOP_ID         2
#define STOP_SEQUENCE   3
#define ARRIVAL_TIME    4
#define DEPARTURE_TIME  5
#define STOP_NAME       6
#define STOP_LAT        7
#define STOP_LON        8
#define SHAPE_ID        9

#define MODE_Aucun      0
#define MODE_Depart     1
#define MODE_Arret      2
#define MODE_Trajet     3

class Dialogue;

class Simulation : public QThread
{
   Q_OBJECT
   public:
      Simulation();
      ~Simulation();
      
      void                  run();
      void                  stop();
      bool                  isStopped();
      int                   getIdBus();
      void                  setRayon(double rayon);
      void                  setTemps(unsigned long tempsArret, unsigned long tempsDeplacement);
      void                  setTrajet(const QVector<QStringList> &trajet, const QVector<QStringList> &trajetDetails);
      void                  setServeurSAI(const QString &adresseIP, int port);
      void                  setId(int idBus, int idService, QString route_id, QString trip_id);
      void                  setAlerte(int codeAlerte);
      void                  simulerTrajet();

   private:
      QVector<QStringList>  trajet;
      QVector<QStringList>  trajetDetails;
      int                   stopSequence;
      int                   stopId;
      unsigned long         tempsArret;
      unsigned long         tempsDeplacement;
      Dialogue             *dialogue;
      QString               adresseIP;
      int                   port;
      int                   idBus;
      int                   idService;
      QString               route_id;
      QString               trip_id;
      QString               delimiteur;
      double                rayon;
      double                distanceParcourue;
      double                duree;
      QTime                 topDepart;
      QTime                 topDepartSimule;
      bool                  stopped;
      QMutex                mutex;
      
      QString               getTrameId(QString type);
      void                  emettreTrameLocalisation(QString latitude, QString longitude, QString cap, QString vitesse);
      void                  emettreTrameArret(QString arret);
      unsigned char         calculerChecksum(QString data);
      int                   getArret(QString latitude, QString longitude);
      double                getCap(QString latitude, QString longitude, QString latitudeS, QString longitudeS);
      double                getVitesse(QString latitudeA, QString longitudeA, QString latitude, QString longitude);
   
   signals:    
      void                  nouvelArret(const QString &arret);
      void                  nouvelArretHorodate(const QString &arret, const QString &horodatage);
      void                  nouvelHorodatage(const QString &horodatage);
      void                  nouvelleDuree(const QString &duree);
      void                  nouvelleDistance(const QString &distance);
      void                  nouvelleVitesse(const QString &vitesse);
      void                  nouvelleTrame(const QString &phrase);
      void                  nouveauMessage(const QString &message);
      void                  trajetFini();

   private slots:
      void                  receptionnerMessage(const QString &message);
};

#endif
