#include <QtGui>

#include "ihm.h"
#include "simulation.h"

IHM::IHM( QWidget *parent ) : QWidget( parent )
{
   labelPortGPS = new QLabel(this);
   labelPortGPS->setText(QString::fromUtf8("Port GPS : "));
   lePortGPS = new QLineEdit(this);
   lePortGPS->setText("/dev/ttyUSB0");
   labelTypePhrase = new QLabel(this);
   labelTypePhrase->setText(QString::fromUtf8("Type : "));
   leTypePhrase = new QComboBox(this);
   leTypePhrase->addItem("GPSINFO");
   leTypePhrase->addItem("NMEA0183");
   leTypePhrase->setCurrentIndex(0);
   messagesSimulateur = new QTextEdit(this);
   messagesSimulateur->setReadOnly(true);

   labelLigne = new QLabel(this);   
   labelLigne->setText(QString::fromUtf8("Ligne : "));
   choixLigne = new QComboBox(this);   
   labelItineraire = new QLabel(this);   
   labelItineraire->setText(QString::fromUtf8("Itinéraire : "));
   choixItineraire = new QComboBox(this);   
   choixItineraire->setEnabled(false);
   vueTrajet = new QTableView(this);

   labelTempsTransit = new QLabel(this);
   labelTempsTransit->setText(QString::fromUtf8("Durée à l'arrêt (ms) : "));
   tempsTransit = new QLineEdit(this);
   tempsTransit->setText("2000");

   labelArret = new QLabel(this);
   labelArret->setFrameStyle(QFrame::Panel | QFrame::Sunken);
   labelArret->setMargin(10);
   labelArret->setText(QString::fromUtf8(" "));
   labelArret->setFont(QFont("Verdana", 14, QFont::Bold));
   labelArret->setAlignment(Qt::AlignHCenter);
   QPalette palette;
   palette.setColor(QPalette::WindowText, Qt::blue);
   labelArret->setAutoFillBackground(true);
   labelArret->setPalette(palette);
   labelHorodatage = new QLabel(this);
   labelHorodatage->setFrameStyle(QFrame::Panel | QFrame::Sunken);
   labelHorodatage->setMargin(10);
   labelHorodatage->setFont(QFont("Verdana", 14, QFont::Bold));
   labelHorodatage->setAlignment(Qt::AlignHCenter);
   labelHorodatage->setText(QString::fromUtf8(" "));
   palette.setColor(QPalette::WindowText, Qt::blue);
   labelHorodatage->setPalette(palette);
   labelDuree = new QLabel(this);
   labelDuree->setFrameStyle(QFrame::Panel | QFrame::Sunken);
   labelDuree->setMargin(10);
   labelDuree->setFont(QFont("Verdana", 14, QFont::Bold));
   labelDuree->setAlignment(Qt::AlignHCenter);
   labelDuree->setText(QString::fromUtf8(" "));
   palette.setColor(QPalette::WindowText, Qt::darkGreen);
   labelDuree->setPalette(palette);
   labelDistance = new QLabel(this);
   labelDistance->setFrameStyle(QFrame::Panel | QFrame::Sunken);
   labelDistance->setMargin(10);
   labelDistance->setFont(QFont("Verdana", 14, QFont::Bold));
   labelDistance->setAlignment(Qt::AlignHCenter);
   labelDistance->setText(QString::fromUtf8(" "));
   palette.setColor(QPalette::WindowText, Qt::darkGreen);
   labelDistance->setPalette(palette);
   labelVitesse = new QLabel(this);
   labelVitesse->setFrameStyle(QFrame::Panel | QFrame::Sunken);
   labelVitesse->setMargin(10);
   labelVitesse->setFont(QFont("Verdana", 14, QFont::Bold));
   labelVitesse->setAlignment(Qt::AlignHCenter);
   labelVitesse->setText(QString::fromUtf8(" "));
   palette.setColor(QPalette::WindowText, Qt::darkGreen);
   labelVitesse->setPalette(palette);

   messagesGPS = new QTextEdit(this);
   messagesGPS->setReadOnly(true);
  
   bDemarrer  = new QPushButton(QString::fromUtf8("Démarrer"), this);  
   bDemarrer->setEnabled(false);
   bArreter   = new QPushButton(QString::fromUtf8("Arrêter"), this);  
   bArreter->setEnabled(false);
   bQuitter   = new QPushButton("Quitter", this);    

   lireParametresINI();

   QHBoxLayout *hLayout0a = new QHBoxLayout;
   QHBoxLayout *hLayout0b = new QHBoxLayout;
   QHBoxLayout *hLayout0 = new QHBoxLayout;
   QHBoxLayout *hLayout1 = new QHBoxLayout;
   QHBoxLayout *hLayout2 = new QHBoxLayout;
   QHBoxLayout *hLayout3 = new QHBoxLayout;
   QHBoxLayout *hLayout4 = new QHBoxLayout;
   QHBoxLayout *hLayout5 = new QHBoxLayout;
   QHBoxLayout *hLayout6 = new QHBoxLayout;
   QHBoxLayout *hLayout7 = new QHBoxLayout;
   QVBoxLayout *mainLayout = new QVBoxLayout;   
   hLayout0a->addWidget(labelPortGPS);
   hLayout0a->addWidget(lePortGPS);
   hLayout0a->addWidget(labelTypePhrase);
   hLayout0a->addWidget(leTypePhrase);
   hLayout0b->addWidget(messagesSimulateur);
   hLayout0->addWidget(labelLigne);
   hLayout0->addWidget(choixLigne);
   hLayout1->addWidget(labelItineraire);
   hLayout1->addWidget(choixItineraire);
   hLayout2->addWidget(vueTrajet);   
   hLayout3->addWidget(labelTempsTransit);
   hLayout3->addWidget(tempsTransit);
   hLayout4->addWidget(labelArret);
   hLayout4->addWidget(labelHorodatage);
   hLayout5->addWidget(labelDuree);
   hLayout5->addWidget(labelDistance);
   hLayout5->addWidget(labelVitesse);
   hLayout6->addWidget(messagesGPS);
   hLayout7->addWidget(bDemarrer);
   hLayout7->addWidget(bArreter);
   hLayout7->addWidget(bQuitter);
   mainLayout->addLayout(hLayout0a);
   mainLayout->addLayout(hLayout0b);
   mainLayout->addLayout(hLayout0);   
   mainLayout->addLayout(hLayout1);
   mainLayout->addLayout(hLayout2);
   mainLayout->addLayout(hLayout3);
   mainLayout->addLayout(hLayout4);
   mainLayout->addLayout(hLayout5);
   mainLayout->addLayout(hLayout6);
   mainLayout->addLayout(hLayout7);
   setLayout(mainLayout);

   initialiserBD();

   simulation = NULL;
   route_id = "";
   trip_id = "";

   connect(this, SIGNAL(signalerMessage(const QString &)), this, SLOT(actualiserMessageSimulateur(const QString &)));
   connect(choixLigne, SIGNAL(currentIndexChanged(int)), this, SLOT(selectionnerLigne(int)));  
   connect(choixItineraire, SIGNAL(currentIndexChanged(int)), this, SLOT(selectionnerItineraire(int)));  
   connect(bDemarrer, SIGNAL(clicked()), this, SLOT(demarrer()));
   connect(bArreter, SIGNAL(clicked()), this, SLOT(arreter()));
   connect(bQuitter, SIGNAL(clicked()), this, SLOT(quitter()));
   connect(this, SIGNAL(quit()), qApp, SLOT(quit()));

   setWindowTitle(QString::fromUtf8("Simulateur GPS Bus"));
   //setFixedHeight(sizeHint().height());
   move(0, 0);
}

void IHM::demarrer() 
{  
    if(simulation == NULL)
    {
       // il faut une ligne et un itinéraire
       if(route_id.isEmpty() || trip_id.isEmpty())
       {
           QMessageBox::critical(0, QObject::tr("Erreur !"), QString::fromUtf8("Impossible de démarrer la simulation !\n%1").arg("Il faut sélectionner une ligne et un itinéraire !"));
           return;
       }

       labelArret->setText("");
       labelHorodatage->setText("");
       labelDuree->setText("");
       labelDistance->setText("");
       labelVitesse->setText("");
       messagesGPS->clear();

       simulation = new Simulation;

       simulation->setType(leTypePhrase->currentText());
       simulation->setTrajet(trajet, trajetDetails);
       simulation->setTemps(tempsTransit->text().toInt(), 500); // milli s
       simulation->setPortGPS(lePortGPS->text());
       simulation->setId(route_id, trip_id);

       connect(simulation, SIGNAL(signalerMessage(const QString &)), this, SLOT(actualiserMessageSimulateur(const QString &)));
       connect(simulation, SIGNAL(nouvelArretHorodate(const QString &, const QString &)), this, SLOT(actualiserArret(const QString &, const QString &)));
       connect(simulation, SIGNAL(nouvelHorodatage(const QString &)), this, SLOT(actualiserHorodatage(const QString &)));
       connect(simulation, SIGNAL(nouvelleDuree(const QString &)), this, SLOT(actualiserDuree(const QString &)));
       connect(simulation, SIGNAL(nouvelleDistance(const QString &)), this, SLOT(actualiserDistance(const QString &)));
       connect(simulation, SIGNAL(nouvelleVitesse(const QString &)), this, SLOT(actualiserVitesse(const QString &)));
       connect(simulation, SIGNAL(nouvelleTrame(const QString &)), this, SLOT(actualiserTrame(const QString &)));
       connect(simulation, SIGNAL(trajetFini()), this, SLOT(arreter()));

       simulation->start();

       lePortGPS->setEnabled(false);
       leTypePhrase->setEnabled(false);
       tempsTransit->setEnabled(false);
       choixLigne->setEnabled(false);
       choixItineraire->setEnabled(false);
       bDemarrer->setEnabled(false);
       bArreter->setEnabled(true);
    }
}

IHM::~IHM()
{
    ecrireParametresINI();
}

void IHM::arreter() 
{      
    if(simulation != NULL)
    {
       if(!simulation->isStopped())
          simulation->stop();
       simulation->wait();

       disconnect(simulation, SIGNAL(signalerMessage(const QString &)), this, SLOT(actualiserMessageSimulateur(const QString &)));
       disconnect(simulation, SIGNAL(nouvelArretHorodate(const QString &, const QString &)), this, SLOT(actualiserArret(const QString &, const QString &)));
       disconnect(simulation, SIGNAL(nouvelHorodatage(const QString &)), this, SLOT(actualiserHorodatage(const QString &)));
       disconnect(simulation, SIGNAL(nouvelleDuree(const QString &)), this, SLOT(actualiserDuree(const QString &)));
       disconnect(simulation, SIGNAL(nouvelleDistance(const QString &)), this, SLOT(actualiserDistance(const QString &)));
       disconnect(simulation, SIGNAL(nouvelleVitesse(const QString &)), this, SLOT(actualiserVitesse(const QString &)));
       disconnect(simulation, SIGNAL(nouvelleTrame(const QString &)), this, SLOT(actualiserTrame(const QString &)));
       disconnect(simulation, SIGNAL(trajetFini()), this, SLOT(arreter()));

       delete simulation;
       simulation = NULL;
       route_id = "";
       trip_id = "";
       trajet.clear();
       trajetDetails.clear();

       lePortGPS->setEnabled(true);
       leTypePhrase->setEnabled(true);
       tempsTransit->setEnabled(true);
       choixLigne->setCurrentIndex(0);
       choixLigne->setEnabled(true);
       itineraires.clear();
       choixItineraire->clear();
       choixItineraire->addItem("");
       choixItineraire->setEnabled(true);
       vueTrajet->setModel(NULL);
       bArreter->setEnabled(false);
       bDemarrer->setEnabled(false);
    }
}

void IHM::quitter() 
{  
   arreter();

   emit quit();
}

void IHM::selectionnerLigne(int index) 
{
   if(index > 0)
   {
      //qDebug() << "ligne choisie :" << lignes.at(index-1);
      QString route_short_name = lignes.at(index-1).at(0);
      
      bool retour;
      QSqlQuery query;
      QString requete = "SELECT routes.route_id, routes.route_short_name, routes.route_long_name, trips.trip_id, trips.trip_headsign, trips.direction_id, stop_times.stop_id, stop_times.stop_sequence, stop_times.departure_time, stops.stop_name, stops.stop_lat, stops.stop_lon FROM routes INNER JOIN trips ON routes.route_id=trips.route_id INNER JOIN stop_times ON stop_times.trip_id=trips.trip_id INNER JOIN stops ON stops.stop_id=stop_times.stop_id WHERE route_short_name=\'" + route_short_name + "\' AND stop_times.stop_sequence=\'0\' ORDER BY departure_time ASC";
      retour = query.exec(requete);
      if(retour)
      {
         itineraires.clear();
         choixItineraire->clear();
         choixItineraire->addItem("");   
         while ( query.next() )  
         {  
            //qDebug() << "enregistrement -> ";
            //for(int i=0;i<query.record().count();i++)
               //qDebug() << query.value(i).toString();
            QStringList itineraire;
            QString route_id = query.value(0).toString();
            itineraire << route_id;			
            QString trip_id = query.value(3).toString();
            itineraire << trip_id;			
            QString trip_headsign = query.value(4).toString();
            itineraire << trip_headsign;			
            QString departure_time = query.value(8).toString();
            itineraire << departure_time;			
            QString stop_name = query.value(9).toString();
            itineraire << stop_name;			
            itineraires.push_back(itineraire);
            
            choixItineraire->addItem(trip_id + " -> " + departure_time + " " + stop_name + " -> " + trip_headsign);   
         } 
         
         choixItineraire->setEnabled(true);
      }
      else  qDebug() << query.lastError().text();
   }
   else
   {
      itineraires.clear();
      choixItineraire->clear();
      choixItineraire->addItem("");   
      choixItineraire->setEnabled(false);
   }
}

void IHM::selectionnerItineraire(int index) 
{
   if(index > 0)
   {
      //qDebug() << "itineraire choisi :" << itineraires.at(index-1);
      route_id = itineraires.at(index-1).at(0);
      trip_id = itineraires.at(index-1).at(1);
      
      bool retour;
      QSqlQuery query;
      //QString requete = "SELECT trips.trip_headsign, trips.direction_id,  stop_times.stop_id, stop_times.stop_sequence, stop_times.arrival_time, stop_times.departure_time, stops.stop_name, stops.stop_lat, stops.stop_lon FROM trips INNER JOIN stop_times ON stop_times.trip_id=trips.trip_id INNER JOIN stops ON stops.stop_id=stop_times.stop_id WHERE trips.trip_id=\'" + trip_id + "\' ORDER BY departure_time ASC";
      QString requete = "SELECT trips.trip_headsign, trips.direction_id, stop_times.stop_id, stop_times.stop_sequence, stop_times.arrival_time, stop_times.departure_time, stops.stop_name, stops.stop_lat, stops.stop_lon, trips.shape_id FROM trips INNER JOIN stop_times ON stop_times.trip_id=trips.trip_id INNER JOIN stops ON stops.stop_id=stop_times.stop_id WHERE trips.trip_id=\'" + trip_id + "\' ORDER BY stop_sequence ASC";
      //qDebug() << requete;
      retour = query.exec(requete);
      if(retour)
      {
         trajet.clear();
         while ( query.next() )  
         {  
            QStringList arret;
            for(int i=0;i<query.record().count();i++)
               arret << query.value(i).toString();         
            trajet.push_back(arret);
         } 
         //qDebug() << "Trajet : " << trajet;
      }
      else  qDebug() << query.lastError().text();
     
      QSqlQueryModel *model = new QSqlQueryModel;
      model->setQuery("SELECT stops.stop_name, stop_times.arrival_time, stop_times.departure_time FROM trips INNER JOIN stop_times ON stop_times.trip_id=trips.trip_id INNER JOIN stops ON stops.stop_id=stop_times.stop_id WHERE trips.trip_id=\'" + trip_id + "\' ORDER BY stop_sequence ASC");
      QTextCodec::setCodecForCStrings(QTextCodec::codecForName("UTF-8"));
      model->setHeaderData(0, Qt::Horizontal, "Arrêt");
      model->setHeaderData(1, Qt::Horizontal, "Arrivée");
      model->setHeaderData(2, Qt::Horizontal, "Départ");
      vueTrajet->setModel(model);
      vueTrajet->resizeColumnsToContents();
      //vueTrajet->show();

      // on récupère le tracé détaillé du parcours

      QString shape_id = trajet.at(0).at(SHAPE_ID);
      if(!shape_id.isEmpty())
      {
        requete = "SELECT * FROM shapes WHERE shape_id=\'" + shape_id + "\' ORDER BY shape_pt_sequence ASC";
        retour = query.exec(requete);
        if(retour)
        {
            trajetDetails.clear();
            while ( query.next() )
            {
               QStringList shapes;
               for(int i=0;i<query.record().count();i++)
                  shapes << query.value(i).toString();
               trajetDetails.push_back(shapes);
            }
            //qDebug() << trajetDetails;
        }
        else
        {
            qDebug() << query.lastError().text();
        }
      }

      bArreter->setEnabled(false);
      bDemarrer->setEnabled(true);
   }
}

void IHM::lireParametresINI()
{
    QString fichierINI = qApp->applicationDirPath() + "/" + qApp->applicationName() + ".ini";
    QSettings parametres(fichierINI, QSettings::IniFormat);
    parametres.setIniCodec(QTextCodec::codecForName("UTF-8"));

    QString portGPS = parametres.value("gps/port", "/dev/ttyUSB0").toString();
    QString type = parametres.value("gps/type", "GPSINFO").toString();

    lePortGPS->setText(portGPS);
    if(type == "GPSINFO")
        leTypePhrase->setCurrentIndex(0);
    else if(type == "NMEA0183")
        leTypePhrase->setCurrentIndex(1);
}

void IHM::ecrireParametresINI()
{
    QString fichierINI = qApp->applicationDirPath() + "/" + qApp->applicationName() + ".ini";
    QSettings parametres(fichierINI, QSettings::IniFormat);
    parametres.setIniCodec(QTextCodec::codecForName("UTF-8"));

    parametres.beginGroup("gps");
    parametres.setValue("port", lePortGPS->text());
    parametres.setValue("type", leTypePhrase->currentText());
    parametres.endGroup();
}

void IHM::initialiserBD()
{
   db = QSqlDatabase::addDatabase("QSQLITE");   
   
   db.setDatabaseName(QCoreApplication::applicationDirPath() + "/referentiel/referentiel-tisseo.sqlite");
   if(!db.open())
   {
      //qDebug() << "referentiel.sqlite erreur !";
      return ;
   }

   QSqlQuery query("SELECT route_short_name,route_long_name FROM routes");
   query.next(); // nom des colonnes
   choixLigne->addItem("");   
   while (query.next()) 
   {
      QStringList route;
      QString route_short_name = query.value(0).toString();
      route << route_short_name;			
      QString route_long_name = query.value(1).toString();
      route << route_long_name;			
      lignes.push_back(route);
      
      //qDebug() << route_short_name << " - " << route_long_name;
      choixLigne->addItem(route_short_name + " - " + route_long_name);
   }
}

void IHM::actualiserArret(const QString &arret, const QString &horodatage)
{
    this->labelArret->setText(arret);
    this->labelHorodatage->setText(horodatage);
}

void IHM::actualiserHorodatage(const QString &horodatage)
{
    this->labelDuree->setText(horodatage);
}

void IHM::actualiserDuree(const QString &duree)
{
    //this->labelDuree->setText(duree);
}

void IHM::actualiserDistance(const QString &distance)
{
    //this->labelDistance->setText(QString().setNum(distance.toDouble(), 'f', 0) + " m");
}

void IHM::actualiserVitesse(const QString &vitesse)
{
    //this->labelVitesse->setText(QString().setNum(vitesse.toDouble(), 'f', 2) + " km/h");
}

void IHM::actualiserTrame(const QString &message)
{
   messagesGPS->append(message);
}

void IHM::actualiserMessageSimulateur(const QString &message)
{
    messagesSimulateur->append(message);
}
