#include "port.h"
#include <unistd.h>
#include <QDebug>

Port::Port():port(NULL)
{
}

Port::Port(const QString &portName):port(NULL)
{
    ouvrir(portName);
}

Port::~Port()
{
    if (port != NULL && port->isOpen())
    {
        port->close();
    }
}

bool Port::ouvrir(const QString &portName)
{
    if (port != NULL && port->isOpen())
    {
        port->close();
    }

    this->port = new QextSerialPort(portName, QextSerialPort::Polling);
    port->setBaudRate(BAUD9600);
    port->setFlowControl(FLOW_OFF);
    port->setParity(PAR_NONE);
    port->setDataBits(DATA_8);
    port->setStopBits(STOP_1);
    //port->setTimeout(500);

    if (port->open(QIODevice::ReadWrite) == true)
    {
        connect(port, SIGNAL(readyRead()), this, SLOT(onReadyRead()));
    }

    return port->isOpen();
}

int Port::ecrire(const QString &trame)
{
    int nombresOctets = -1;

    if (port == NULL || !port->isOpen())
    {
        return -1;
    }

    nombresOctets = port->write(trame.toLatin1());
    #ifdef DEGUG_PORT
    qDebug() << Q_FUNC_INFO << "nombresOctets =" << nombresOctets;
    qDebug() << Q_FUNC_INFO << "trame =" << trame;
    #ifdef DEGUG_PORT_1
    for(int i = 0 ; i < trame.size() ; i++)
    {
        printf("0x%02X ", (unsigned char)trame.at(i).toAscii());
    }    
    printf("\n\n");
    #endif
    #endif

    return nombresOctets;
}

int Port::ecrire(const QString trame, int nb)
{
    Q_UNUSED(nb)
    int nombresOctets = -1;

    if (port == NULL || !port->isOpen())
    {
        return -1;
    }

    nombresOctets = port->write(trame.toLatin1());
    //nombresOctets = port->write(trame, nb);

    #ifdef DEGUG_PORT
    qDebug() << Q_FUNC_INFO << "nombresOctets =" << nombresOctets;
    qDebug() << Q_FUNC_INFO << "trame =" << trame;
    #ifdef DEGUG_PORT_1
    for(int i = 0 ; i < trame.size() ; i++)
    {
        printf("0x%02X ", (unsigned char)trame.at(i).toAscii());
    }
    printf("\n\n");
    #endif
    #endif

    return nombresOctets;
}

int Port::recevoir(QString &message)
{
    char buffer[1024+1]; // + fin de chaine
    int nombresOctets = 16;

    if (port == NULL || !port->isOpen())
    {
        return -1;
    }

    /*nombresOctets = port->bytesAvailable();
    if(nombresOctets > 1024)
    {
        nombresOctets = 1024;
    }
    #ifdef DEGUG_PORT
    if(nombresOctets != 0)
    {
        qDebug() << Q_FUNC_INFO << "nombresOctets =" << nombresOctets;
    }
    #endif
    */

    bool fini = false;
    int n = 0;
    int nb = 0;
    do
    {
        if(port->bytesAvailable())
        {
            n += port->read(&buffer[n], nombresOctets);
            if (n > 0)
            {
                if(buffer[n-1] == '\n')
                    fini = true;
                buffer[n] = '\0'; // fin de chaine
            }
        }
        else
        {
            usleep(200000);
            nb++;
        }
    }
    while(!fini && nb < 3);

    if(nb != 3)
        message = QLatin1String(buffer);
    else
    {
        message = QLatin1String("");
        n = 0;
    }

    #ifdef DEGUG_PORT
    if(n != 0)
    {
        qDebug() << Q_FUNC_INFO << "n =" << n;
        qDebug() << Q_FUNC_INFO << "message =" << message;
    }
    #ifdef DEGUG_PORT_1
    for(int i = 0 ; i < n ; i++)
    {
        printf("0x%02X ", (unsigned char)buffer[i]);
    }

    printf("\n\n");
    #endif
    #endif

    return n;
}

char Port::lire()
{
    char caractereLu;

    if (port == NULL || !port->isOpen())
    {
        return -1;
    }

    int nombresOctets = port->read(&caractereLu, 1);
    if (nombresOctets == -1)
    {
        caractereLu = '\0'; // fin de chaine
    }

    #ifdef DEGUG_PORT
    qDebug() << Q_FUNC_INFO << "caractereLu =" << caractereLu;
    qDebug("char Port::lire() caractereLu = 0x%02X", caractereLu);
    #endif

    return caractereLu;
}

long Port::nbOctetsDisponibles()
{
    return port->bytesAvailable();
}

void Port::onReadyRead()
{
    QByteArray donnees;
    int nombresOctets = port->bytesAvailable();
    donnees.resize(nombresOctets);
    port->read(donnees.data(), donnees.size());

    #ifdef DEGUG_PORT
    qDebug() << Q_FUNC_INFO << "nombresOctets =" << nombresOctets;
    qDebug() << Q_FUNC_INFO << "donnees =" << donnees;
    #ifdef DEGUG_PORT_1
    for(int i = 0 ; i < donnees.size() ; i++)
    {
        printf("0x%02X ", (unsigned char)donnees.data()[i]);
    }    
    printf("\n\n");
    #endif
    #endif
}

bool Port::estOuvert()
{
    return port->isOpen();
}

qint64 Port::read(char * data, qint64 maxSize)
{
    return port->read(data, maxSize);
}

qint64 Port::write(const char * data, qint64 maxSize)
{
    return port->write(data, maxSize);
}

qint64 Port::write(const char * data)
{
    return port->write(data);
}
