#include "simulation.h"
#include "geolocalisation.h"
#include "port.h"

/*
AT+CGPS=1,1

OK
AT+CGPSINFO

+CGPSINFO:,,,,,,,,

OK
AT+CGPSINFO

+CGPSINFO:4356.925885,N,00448.776406,E,040516,112824.0,90.0,0,0

OK
AT+CGPS=1,1

+CGPSINFO:4356.925890,N,00448.776416,E,040516,112846.0,90.0,0,0

OK
*/

Simulation::Simulation()
{   
   stopped = false;
   stopSequence = 0;
   stopId = 0;
   tempsArret = 0;
   tempsDeplacement = 0;
   delimiteur = ",";
   rayon = 40.;
   duree = 0;
   demarrageGPS = false;
   reponseGPSINFO = false;
}

Simulation::~Simulation()
{    
}

void Simulation::run()
{
   this->setPriority(QThread::NormalPriority);
   
   port = new Port(portGPS);
   if(port->estOuvert())
        emit signalerMessage(QString::fromUtf8("port ") + portGPS + QString::fromUtf8(" ouvert"));
   else
        emit signalerMessage("<b><font color=red>" + QString::fromUtf8("erreur ouverture port ") + portGPS + QString::fromUtf8(" !") + "</font></b>");

   qDebug() << Q_FUNC_INFO << "demarrage ...\n";
   emit signalerMessage(QString::fromUtf8("démarrage simulation"));
   try
   {
      distanceParcourue = 0.;
      topDepart = QTime::currentTime();
      QString heureDepart = trajet.at(0).at(DEPARTURE_TIME);
      topDepartSimule = QTime::fromString(heureDepart, "hh:mm:ss");

      while(!isStopped())
      {                      
            simulerTrajet();
            // prochain arrêt
            QThread::usleep(tempsArret);
            duree += tempsArret/1000000.;          
      }
   }
   catch (...)
   {
      // faire quelque chose avec les exceptions
   }
   QMutexLocker verrou_stop(&mutex);
   stopped = false;
   
   emit trajetFini();
   emit signalerMessage(QString::fromUtf8("arrêt simulation"));
   delete port;
   qDebug() << Q_FUNC_INFO << "terminee.\n";
}

void Simulation::stop()
{
    QMutexLocker verrou_stop(&mutex);
    stopped = true;
}

bool Simulation::isStopped()
{
    QMutexLocker verrou_stop(&mutex);
    return stopped;
}

void Simulation::setType(const QString &typePhrase)
{
    this->typePhrase = typePhrase;
}

void Simulation::setTemps(unsigned long tempsArret, unsigned long tempsDeplacement)
{
    // milli s -> micro s
    this->tempsArret = tempsArret * 1000;
    this->tempsDeplacement = tempsDeplacement * 1000;
}

void Simulation::setTrajet(const QVector<QStringList> &trajet, const QVector<QStringList> &trajetDetails)
{
   this->trajet = trajet;
   this->trajetDetails = trajetDetails;
}

void Simulation::setPortGPS(const QString &portGPS)
{
    this->portGPS = portGPS;
}

void Simulation::setId(QString route_id, QString trip_id)
{
    this->route_id = route_id;
    this->trip_id = trip_id;
}

void Simulation::simulerTrajet()
{
    QString arret;
    QString latitude, longitude;
    QString latitudeS, longitudeS;
    QString latitudeA, longitudeA;
    int id;
    double cap, vitesse;
    QString message;
    int n;

    //qDebug() << Q_FUNC_INFO << trajet;
    //qDebug() << Q_FUNC_INFO << trajetDetails;
    for (int i = stopId; i < trajetDetails.size();)
    {
        //qDebug() << Q_FUNC_INFO << __LINE__ << stopId << i << trajetDetails.size() << id << trajet.size() << stopSequence;
        n = port->recevoir(message);
        if(n > 0)
            traiterMessage(message);

        if(demarrageGPS == false)
            continue;

        latitude = trajetDetails.at(i).at(1);
        longitude = trajetDetails.at(i).at(2);
        id = getArret(latitude, longitude);

        if((i + 1) < trajetDetails.size())
        {
            latitudeS = trajetDetails.at(i+1).at(1);
            longitudeS = trajetDetails.at(i+1).at(2);
            cap = getCap(latitude, longitude, latitudeS, longitudeS);
        }
        else
            cap = 0.;

        if(i != 0)
        {
            latitudeA = trajetDetails.at(i-1).at(1);
            longitudeA = trajetDetails.at(i-1).at(2);
            vitesse = getVitesse(latitudeA, longitudeA, latitude, longitude);
        }
        else
            vitesse = 0.;

        emettreTrameLocalisation(latitude, longitude, QString::number(cap), QString::number(vitesse));

        n = port->recevoir(message);
        if(n > 0)
            traiterMessage(message);

        if(id != -1)
        {
            //qDebug() << Q_FUNC_INFO << latitude << longitude;
            //if(trajet.at(id).at(STOP_SEQUENCE) == QString::number(stopSequence))            
            {
                if(id < trajet.size())
                {
                    arret = trajet.at(id).at(STOP_NAME);
                    latitude = trajet.at(id).at(STOP_LAT);
                    longitude = trajet.at(id).at(STOP_LON);

                    emettreTrameLocalisation(latitude, longitude, QString::number(cap), QString::number(vitesse));

                    QTime maintenantSimule;
                    maintenantSimule = topDepartSimule.addSecs((int)duree*10);
                    emit nouvelArretHorodate(arret, maintenantSimule.toString("hh:mm:ss"));
                    stopSequence++;
                }
                stopId = i;

                return;
            }
        }

        // prochaine localisation
        QThread::usleep(tempsDeplacement);
        duree += tempsDeplacement/1000000.;
        ++i;
    }

    qDebug() << Q_FUNC_INFO << "trajet fini";
    emit trajetFini();
    stopped = true;
}

QString Simulation::getTrameEntete(QString type)
{
   QString entete;
   QString heure;
   QTime maintenant;
   QDate aujourdhui;
   QTime maintenantSimule;

   maintenantSimule = topDepartSimule.addSecs((int)duree*10);

   if(typePhrase == "GPSINFO")
   {
       /*
        * 1 - standalone mode
        * 2 - UE-based mode
        * 3 - UE-assisted mode
        *
        * If UE-assisted mode, when fixed will report indication:
        * +CAGPSINFO:<lat>,<lon>,<alt>,<date>,<time>
        *
        * +CGPSINFO: [<lat>],[<N/S>],[<log>],[<E/W>],[<date>],[<UTCtime>],[<alt>],[<speed>],[<course>]
        * [+CGPSINFO: <RMC>]
        *
        *   <lat>       Output format is ddmm.mmmm
        *   <log>       Output format is dddmm.mmmm
        *   <date>      Output format is ddmmyy
        *   <UTC time>  Output format is hhmmss.s
        *   <alt>       Unit is meters.
        *   <speed>     Unit is knots.
        *   <course>    Degrees.
        *
        * Exemple : +CGPSINFO:3113.343286,N,12121.234064,E,250311,072809.3,44.1,0.0,0
        *
        */

       heure = maintenantSimule.toString("hhmmss");
       //heure.replace(":", "");
       entete = "AT+CGPSINFO\r\r\n+CGPSINFO:";
       //entete = "+CGPSINFO:";
   }
   else if(typePhrase == "NMEA0183")
   {
        heure = maintenantSimule.toString("hhmmss");
        //heure.replace(":", "");
        entete = "$GP" + type + delimiteur + heure + ".000" + delimiteur;
   }

   //qDebug() << Q_FUNC_INFO << "entete = " << entete;   

   return entete;
}

void Simulation::emettreTrameLocalisation(QString latitude, QString longitude, QString cap, QString vitesse)
{
    QDate aujourdhui;
    QString phrase;
    unsigned char checksum;
    QString strChecksum;
    QString trame;
    QTime maintenantSimule;    

    maintenantSimule = topDepartSimule.addSecs((int)duree*10);
    emit nouvelHorodatage(maintenantSimule.toString("hh:mm:ss"));

    if(typePhrase == "GPSINFO")
    {
        if(demarrageGPS == false)
        {
            if(reponseGPSINFO == true)
            {
				//this->msleep(10);
                phrase += QString("AT+CGPSINFO\r\r\n+CGPSINFO:,,,,,,,,\r\nOK\r\n");
                port->ecrire(phrase);
                phrase.remove(phrase.size()-2, 2); // enlève le \r\n pour l'ihm
                trame = "<b><font color=blue>" + phrase + "</font></b>";
                emit nouvelleTrame(trame);
                reponseGPSINFO = false;
            }
            return;
        }

        if(reponseGPSINFO == false)
        {
            return;
        }

        /* +CGPSINFO: [<lat>],[<N/S>],[<log>],[<E/W>],[<date>],[<UTCtime>],[<alt>],[<speed>],[<course>]
        * [+CGPSINFO: <RMC>]
        *
        *   <lat>       Output format is ddmm.mmmm
        *   <log>       Output format is dddmm.mmmm
        *   <date>      Output format is ddmmyy
        *   <UTC time>  Output format is hhmmss.s
        *   <alt>       Unit is meters.
        *   <speed>     Unit is knots.
        *   <course>    Degrees.
        *
        * Exemple : +CGPSINFO:3113.343286,N,12121.234064,E,250311,072809.3,44.1,0.0,0
        *
        */
        phrase = getTrameEntete("");
        phrase += convertirLatitude(latitude) + delimiteur + "N" + delimiteur + convertirLongitude(longitude) + delimiteur + "E" + delimiteur;
        phrase += aujourdhui.currentDate().toString("ddMMyy") + delimiteur + maintenantSimule.toString("hhmmss") + ".0" + delimiteur + "44.9" + delimiteur + "0.0" + delimiteur + "0";
        phrase += QString("\r\nOK\r\n");

        port->ecrire(phrase);
        phrase.remove(phrase.size()-2, 2); // enlève le \r\n pour l'ihm
        trame = "<b><font color=green>" + phrase + "</font></b>";
        emit nouvelleTrame(trame);
        emit nouvelHorodatage(maintenantSimule.toString("hh:mm:ss"));
        reponseGPSINFO = false;
    }
    else if(typePhrase == "NMEA0183")
    {
        /*
        $GPGGA,111841.000,4405.1958,N,00457.8627,E,1,05,2.54,44.9,M,49.1,M,,*5E
        $GPGSA,A,3,06,20,07,10,09,,,,,,,,2.71,2.54,0.95*02
        $GPRMC,111841.000,A,4405.1958,N,00457.8627,E,0.76,225.51,240115,,,A*6C
        $GPVTG,225.51,T,,M,0.76,N,1.40,K,A*38
        ...
        $GPGSV,3,1,12,20,79,354,16,23,74,056,,09,71,233,16,06,54,298,25*79
        */

        //Exemple : $GPGGA,111841.000,4405.1958,N,00457.8627,E,1,05,2.54,44.9,M,49.1,M,,*5E
        phrase = getTrameEntete("GGA");
        phrase += convertirLatitude(latitude) + delimiteur + "N" + delimiteur + convertirLongitude(longitude) + delimiteur + "E" + delimiteur;
        phrase += "1" + delimiteur + "05" + delimiteur + "2.54" + delimiteur + "44.9" + delimiteur + "M" + delimiteur + "49.1" + delimiteur + "M" + delimiteur + "" + delimiteur + "*";
        checksum = calculerChecksum(phrase);
        strChecksum = QString("%1").arg(checksum, 2, 16, QLatin1Char('0'));
        phrase += strChecksum.toUpper();
        phrase += QString("\r\n");
        port->ecrire(phrase);
        phrase.remove(phrase.size()-2, 2); // enlève le \r\n pour l'ihm
        trame = "<b><font color=green>" + phrase + "</font></b>";
        emit nouvelleTrame(trame);
        emit nouvelHorodatage(maintenantSimule.toString("hh:mm:ss"));

        //Exemple : $GPRMC,111841.000,A,4405.1958,N,00457.8627,E,0.76,225.51,240115,,,A*6C
        phrase = getTrameEntete("RMC");
        phrase += "A" + delimiteur + convertirLatitude(latitude) + delimiteur + "N" + delimiteur + convertirLongitude(longitude) + delimiteur + "E" + delimiteur;
        phrase += "0.0" + delimiteur + cap + delimiteur + aujourdhui.currentDate().toString("ddMMyy") + delimiteur + "" + delimiteur + "" + delimiteur + "A" + "*";
        checksum = calculerChecksum(phrase);
        strChecksum = QString("%1").arg(checksum, 2, 16, QLatin1Char('0'));
        phrase += strChecksum.toUpper();
        phrase += QString("\r\n");
        port->ecrire(phrase);
        phrase.remove(phrase.size()-2, 2); // enlève le \r\n pour l'ihm
        trame = "<b><font color=green>" + phrase + "</font></b>";
        emit nouvelleTrame(trame);
        emit nouvelHorodatage(maintenantSimule.toString("hh:mm:ss"));
    }
}

unsigned char Simulation::calculerChecksum(QString data)
{
   unsigned char checksum = 0;   
   
   for(int i=1;i<data.length() && data.at(i).toAscii() != '*';i++)
   {
      //printf("0x%02X ", data.at(i).toAscii());
      checksum ^= data.at(i).toAscii(); // XOR
   }
   //printf("\n");   
   //qDebug("(checksum) 0x%02X\n", checksum);
      
   return checksum;
}

int Simulation::getArret(QString latitude, QString longitude)
{
    int id = -1;
    QString latitudeA;
    QString longitudeA;
    double distance = 0.;
    double distanceMin = 10000;
    Geolocalisation *point1, *point2;

    point1 = new Geolocalisation(latitude.toDouble(), longitude.toDouble()*-1);
    int i = stopSequence;
    //qDebug() << Q_FUNC_INFO << "*** recherche arret";
    //for (int i = stopSequence; i < trajet.size(); ++i)
    {
        if(i < trajet.size())
        {
            latitudeA = trajet.at(i).at(STOP_LAT);
            longitudeA = trajet.at(i).at(STOP_LON);
            point2 = new Geolocalisation(latitudeA.toDouble(), longitudeA.toDouble()*-1);
            distance = point1->distance(*point2) * 1000;
            if(distance < distanceMin)
                distanceMin = distance;
            if(distance < rayon)
            {
                //qDebug() << Q_FUNC_INFO << "### distance =" << distance;
                qDebug() << Q_FUNC_INFO << "### arret =" << trajet.at(i).at(STOP_NAME) << "- stop sequence =" << trajet.at(i).at(STOP_SEQUENCE) << "- distance =" << distance;
                return trajet.at(i).at(STOP_SEQUENCE).toInt();
            }
        }
    }
    //qDebug() << Q_FUNC_INFO << "*** distance min =" << distanceMin;
    //qDebug() << Q_FUNC_INFO << "*** arret non trouve";
    return id;
}

double Simulation::getCap(QString latitude, QString longitude, QString latitudeS, QString longitudeS)
{
    double orientation = 0.;
    Geolocalisation *point1, *point2;

    point1 = new Geolocalisation(latitude.toDouble(), longitude.toDouble()*-1);
    point2 = new Geolocalisation(latitudeS.toDouble(), longitudeS.toDouble()*-1);
    orientation = point1->orientation(*point2);

    //qDebug() << Q_FUNC_INFO << "orientation = " << orientation;

    return orientation;
}

double Simulation::getVitesse(QString latitudeA, QString longitudeA, QString latitude, QString longitude)
{
    double distance, vitesse = 0.;
    Geolocalisation *point1, *point2;
    QTime maintenantSimule;
    //QTime horodatage = QTime::currentTime();

    maintenantSimule = topDepartSimule.addSecs((int)duree);

    point1 = new Geolocalisation(latitudeA.toDouble(), longitudeA.toDouble()*-1);
    point2 = new Geolocalisation(latitude.toDouble(), longitude.toDouble()*-1);
    distance = point1->distance(*point2) * 1000;
    distanceParcourue += distance;
    //vitesse = (distanceParcourue / 1000.) / ((double)topDepart.secsTo(horodatage)*5. / 3600.);
    //vitesse = (distanceParcourue / 1000.) / ((double)topDepartSimule.secsTo(maintenantSimule) / 3600.);
    vitesse = (distance / 1000.) / ((double)topDepartSimule.secsTo(maintenantSimule) / 3600.);

    //qDebug() << Q_FUNC_INFO << "intervalle = " << topDepartSimule.secsTo(maintenantSimule);
    //qDebug() << Q_FUNC_INFO << "distance = " << distance;
    //qDebug() << Q_FUNC_INFO << "distance parcourue = " << distanceParcourue;
    //qDebug() << Q_FUNC_INFO << "vitesse = " << vitesse;
    //qDebug();

    emit nouvelleDistance(QString::number(distanceParcourue));
    emit nouvelleVitesse(QString::number(vitesse));

    return vitesse;
}

QString Simulation::convertirLatitude(QString latitude)
{
    int dd = (int)latitude.toDouble();
    double mm = latitude.toDouble() - (double)dd;
    char conversion[16];

    //Conversion degré décimal dd.dddd en format GPS ddmm.mmmm
    //dd.dddd -> dd (int)
    //0.dddd x 60 -> mm.mmmm
    sprintf(conversion, "%02d%02.4f", dd, (float)mm*60.);
    QString latitudeConvertie(conversion);

    latitudeConvertie.replace(",", ".");

    return latitudeConvertie;
}

QString Simulation::convertirLongitude(QString longitude)
{
    int dd = (int)longitude.toDouble();
    double mm = longitude.toDouble() - (double)dd;
    char conversion[16];

    //Conversion degré décimal dd.dddd en format GPS dddmm.mmmm
    //dd.dddd -> ddd (int)
    //0.dddd x 60 -> mm.mmmm
    //1.4611816 -> 0127.6709

    sprintf(conversion, "%03d%02.4f", dd, (float)mm*60.0);
    QString longitudeConvertie(conversion);

    longitudeConvertie.replace(",", ".");

    return longitudeConvertie;
}

void Simulation::traiterMessage(QString &message)
{
    QString phrase;
    QString trame;

    //qDebug() << Q_FUNC_INFO << message;
    // démarrage
    if(message.contains("AT+CGPS=1,1"))
    {
        demarrageGPS = true;
        /*
        AT+CGPS=1,1

        OK
        */
        phrase = QString("AT+CGPS=1,1\r\r\nOK\r\n");
        port->ecrire(phrase);
        phrase.remove(phrase.size()-2, 2); // enlève le \r\n pour l'ihm
        trame = "<b><font color=red>" + phrase + "</font></b>";
        emit nouvelleTrame(trame);
        return;
    }
    // éteindre
    if(message.contains("AT+CGPS=0,1"))
    {
        demarrageGPS = false;
        /*
        AT+CGPS=0,1

        OK
        */
        phrase = QString("AT+CGPS=0,1\r\r\nOK\r\n");
        port->ecrire(phrase);
        phrase.remove(phrase.size()-2, 2); // enlève le \r\n pour l'ihm
        trame = "<b><font color=red>" + phrase + "</font></b>";
        emit nouvelleTrame(trame);
        return;
    }
    // demande
    if(message.contains("AT+CGPSINFO"))
    {
        reponseGPSINFO = true;
        return;
    }
    //qDebug() << Q_FUNC_INFO << demarrageGPS << reponseGPSINFO;

    phrase = QString("ERROR\r\n");
    port->ecrire(phrase);
    phrase.remove(phrase.size()-2, 2); // enlève le \r\n pour l'ihm
    trame = "<b><font color=black>" + phrase + "</font></b>";
    emit nouvelleTrame(trame);
}
