#ifndef GPS_H
#define GPS_H

#include <QtCore>
#include <QTime>
#include "qextserialenumerator.h"
#include "qextserialport.h"

using namespace std;

class Port;

/**
 * @brief Classe qui permet l'aquisition et le traitement des données de géolocalisation
 *
 * @class GPS gps.h "gps.h"
 *
 * @author Rémy Roux <remy.roux84130@gmail.com>
 */
class GPS : public QObject
{
    Q_OBJECT

private:
    Port *port; /**< Pointeur sur un objet Port */
    bool etatDemarre; /**< Un bool qui permet de vérifier l'état du gps (demarré/éteind) */
    QString latitudeBus; /**< Un QString qui contient la latitude du bus */
    QString longitudeBus; /**< Un QString qui contient la longitude du bus */

    /**
     * @brief Méthode qui permet de transmettre des commandes AT au module sim5218e
     *
     * @fn transmettre
     * @param message
     * @return int
     */
    int transmettre(const QString &message);
    /**
     * @brief Méthode qui permet de recevoir les réponses du module sim5218e
     *
     * @fn recevoir
     * @param timeout
     * @return QString
     */
    QString recevoir(unsigned int timeout);

public:
/**
 * @brief Constructeur par défaut de la classe GPS
 *
 * @fn GPS
 */
    GPS();
    /**
     * @brief Destructeur par défaut de la classe GPS
     *
     * @fn ~GPS
     */
    ~GPS();
    /**
     * @brief Méthode qui permet de démarrer le GPS
     *
     * @fn demarrerGPS
     * @return bool
     */
    bool demarrerGPS();
    /**
     * @brief Méthode qui permet d'éteindre le GPS
     *
     * @fn eteindreGPS
     * @return bool
     */
    bool eteindreGPS();
    /**
     * @brief Méthode qui permet de lire et de découper la trame gps
     *
     * @fn lire
     * @return QStringList
     */
    QStringList lire();
    /**
     * @brief Méthode qui permet d'extraire la longitude et la latitude de la trame et de convertir la longitude et la latitude en données exploitables
     *
     * @fn traiterInfosGPS
     * @param infos
     */
    void traiterInfosGPS(QStringList infos);
    /**
     * @brief Accesseur get qui permet de récuper la latitude
     *
     * @fn getLatitude
     * @return QString
     */
    QString getLatitude();
    /**
     * @brief Accesseur get qui permet de récuper la longitude
     *
     * @fn getLongitude
     * @return QString
     */
    QString getLongitude();

public slots:

signals:


};

#endif // GPS_H
